@@ -17,10 +17,12 @@
 #  limitations under the License.
 ###############################################################################
 
+import cgi
 import cherrypy
 import collections
 import datetime
 import json
+import posixpath
 import six
 import sys
 import traceback
@@ -28,10 +30,9 @@ import traceback
 from . import docs
 from girder import events, logger, logprint
 from girder.constants import SettingKey, TokenScope, SortDir
-from girder.models.model_base import AccessException, GirderException, \
-    ValidationException
+from girder.models.model_base import AccessException, GirderException, ValidationException
 from girder.utility.model_importer import ModelImporter
-from girder.utility import config, JsonEncoder
+from girder.utility import toBool, config, JsonEncoder
 from six.moves import range, urllib
 
 # Arbitrary buffer length for stream-reading request bodies
@@ -59,21 +60,34 @@ def getUrlParts(url=None):
     return urllib.parse.urlparse(url)
 
 
-def getApiUrl(url=None):
+def getApiUrl(url=None, preferReferer=False):
     """
     In a request thread, call this to get the path to the root of the REST API.
     The returned path does *not* end in a forward slash.
 
     :param url: URL from which to extract the base URL. If not specified, uses
-        `cherrypy.url()`
+        the server root system setting. If that is not specified, uses `cherrypy.url()`
+    :param preferReferer: if no url is specified, this is true, and this is in
+        a cherrypy request that has a referer header that contains the api
+        string, use that referer as the url.
     """
+    apiStr = '/api/v1'
+
+    if not url:
+        if preferReferer and apiStr in cherrypy.request.headers.get('referer', ''):
+            url = cherrypy.request.headers['referer']
+        else:
+            root = ModelImporter.model('setting').get(SettingKey.SERVER_ROOT)
+            if root:
+                return posixpath.join(root, config.getConfig()['server']['api_root'].lstrip('/'))
+
     url = url or cherrypy.url()
-    idx = url.find('/api/v1')
+    idx = url.find(apiStr)
 
     if idx < 0:
         raise GirderException('Could not determine API root in %s.' % url)
 
-    return url[:idx + 7]
+    return url[:idx + len(apiStr)]
 
 
 def iterBody(length=READ_BUFFER_LEN, strictLength=False):
@@ -133,9 +147,9 @@ def _cacheAuthUser(fun):
 
         user = fun(returnToken, *args, **kwargs)
         if isinstance(user, tuple):
-            setattr(cherrypy.request, 'girderUser', user[0])
+            setCurrentUser(user[0])
         else:
-            setattr(cherrypy.request, 'girderUser', user)
+            setCurrentUser(user)
 
         return user
     return inner
@@ -227,6 +241,17 @@ def getCurrentUser(returnToken=False):
         return retVal(user, token)
 
 
+def setCurrentUser(user):
+    """
+    Explicitly set the user for the current request thread. This can be used
+    to enable specialized auth behavior on a per-request basis.
+
+    :param user: The user to set as the current user of this request.
+    :type user: dict or None
+    """
+    cherrypy.request.girderUser = user
+
+
 def requireAdmin(user, message=None):
     """
     Calling this on a user will ensure that they have admin rights.  If not,
@@ -266,6 +291,26 @@ def getBodyJson(allowConstants=False):
         raise RestException('Invalid JSON passed in request body.')
 
 
+def getParamJson(name, params, default=None):
+    """
+    For parameters that are expected to be specified as JSON, use
+    this to parse them, or raises a RestException if parsing fails.
+
+    :param name: The param name.
+    :type name: str
+    :param params: The dictionary of parameters.
+    :type params: dict
+    :param default: The default value if no such param was passed.
+    """
+    if name not in params:
+        return default
+
+    try:
+        return json.loads(params[name])
+    except ValueError:
+        raise RestException('The %s parameter must be valid JSON.' % name)
+
+
 class loadmodel(ModelImporter):  # noqa: class name
     """
     This is a decorator that can be used to load a model based on an ID param.
@@ -289,9 +334,11 @@ class loadmodel(ModelImporter):  # noqa: class name
     :param exc: Whether an exception should be raised for a nonexistent
         resource.
     :type exc: bool
+    :param requiredFlags: Access flags that are required on the object being loaded.
+    :type requiredFlags: str or list/set/tuple of str or None
     """
     def __init__(self, map=None, model=None, plugin='_core', level=None,
-                 force=False, exc=True, **kwargs):
+                 force=False, exc=True, requiredFlags=None, **kwargs):
         if map is None:
             self.map = {'id': model}
         else:
@@ -303,6 +350,7 @@ class loadmodel(ModelImporter):  # noqa: class name
         self.plugin = plugin
         self.exc = exc
         self.kwargs = kwargs
+        self.requiredFlags = requiredFlags
 
     def _getIdValue(self, kwargs, idParam):
         if idParam in kwargs:
@@ -334,6 +382,10 @@ class loadmodel(ModelImporter):  # noqa: class name
                     raise RestException(
                         'Invalid %s id (%s).' % (model.name, str(id)))
 
+                if self.requiredFlags:
+                    model.requireAccessFlags(
+                        kwargs[converted], user=getCurrentUser(), flags=self.requiredFlags)
+
             return fun(*args, **kwargs)
         return wrapped
 
@@ -353,7 +405,7 @@ class filtermodel(ModelImporter):  # noqa: class name
         :param addFields: Extra fields (key names) that should be included in
             the returned document(s), in addition to any in the model's normal
             whitelist. Only affects top level fields.
-        :type addFields: set, list, tuple, or None
+        :type addFields: `set, list, tuple, or None`
         """
         self.modelName = model
         self.plugin = plugin
@@ -434,8 +486,9 @@ def _createResponse(val):
         elif accept.value == 'text/html':  # pragma: no cover
             # Pretty-print and HTML-ify the response for the browser
             setResponseHeader('Content-Type', 'text/html')
-            resp = json.dumps(val, indent=4, sort_keys=True, allow_nan=False,
-                              separators=(',', ': '), cls=JsonEncoder)
+            resp = cgi.escape(json.dumps(
+                val, indent=4, sort_keys=True, allow_nan=False, separators=(',', ': '),
+                cls=JsonEncoder))
             resp = resp.replace(' ', '&nbsp;').replace('\n', '<br />')
             resp = '<div style="font-family:monospace;">%s</div>' % resp
             return resp.encode('utf8')
@@ -557,14 +610,14 @@ def ensureTokenScopes(token, scope):
     :param token: The token object used in the request.
     :type token: dict
     :param scope: The required scope or set of scopes.
-    :type scope: str or list of str
+    :type scope: `str or list of str`
     """
     tokenModel = ModelImporter.model('token')
     if tokenModel.hasScope(token, TokenScope.USER_AUTH):
         return
 
     if not tokenModel.hasScope(token, scope):
-        setattr(cherrypy.request, 'girderUser', None)
+        setCurrentUser(None)
         if isinstance(scope, six.string_types):
             scope = (scope,)
         raise AccessException(
@@ -779,7 +832,7 @@ class Resource(ModelImporter):
         :param method: The HTTP method of the current request.
         :type method: str
         :param path: The path params of the request.
-        :type path: list
+        :type path: `list`
         """
         if not self._routes:
             raise Exception('No routes defined for resource')
@@ -853,9 +906,9 @@ class Resource(ModelImporter):
         wildcard tokens of the route.
 
         :param path: The requested path.
-        :type path: list
+        :type path: `list`
         :param route: The route specification to match against.
-        :type route: list
+        :type route: `list`
         """
         wildcards = {}
         for i in range(0, len(route)):
@@ -865,46 +918,54 @@ class Resource(ModelImporter):
                 return False
         return wildcards
 
-    def requireParams(self, required, provided):
+    def requireParams(self, required, provided=None):
         """
-        Throws an exception if any of the parameters in the required iterable
-        is not found in the provided parameter set.
+        This method has two modes. In the first mode, this takes two
+        parameters, the first being a required parameter or list of
+        them, and the second the dictionary of parameters that were
+        passed. If the required parameter does not appear in the
+        passed parameters, a ValidationException is raised.
+
+        The second mode of operation takes only a single
+        parameter, which is a dict mapping required parameter names
+        to passed in values for those params. If the value is ``None``,
+        a ValidationException is raised. This mode works well in conjunction
+        with the ``autoDescribeRoute`` decorator, where the parameters are
+        not all contained in a single dictionary.
 
         :param required: An iterable of required params, or if just one is
             required, you can simply pass it as a string.
-        :type required: list, tuple, or str
+        :type required: `list, tuple, or str`
         :param provided: The list of provided parameters.
         :type provided: dict
         """
-        if isinstance(required, six.string_types):
-            required = (required,)
+        if provided is None and isinstance(required, dict):
+            for name, val in six.viewitems(required):
+                if val is None:
+                    raise RestException('Parameter "%s" is required.' % name)
+        else:
+            if isinstance(required, six.string_types):
+                required = (required,)
 
-        for param in required:
-            if param not in provided:
-                raise RestException("Parameter '%s' is required." % param)
+            for param in required:
+                if provided is None or param not in provided:
+                    raise RestException('Parameter "%s" is required.' % param)
 
     def boolParam(self, key, params, default=None):
         """
-        Coerce a parameter value from a str to a bool. This function is case
-        insensitive. The following string values will be interpreted as True:
+        Coerce a parameter value from a str to a bool.
 
-          - ``'true'``
-          - ``'on'``
-          - ``'1'``
-          - ``'yes'``
-
-        All other strings will be interpreted as False. If the given param
-        is not passed at all, returns the value specified by the default arg.
+        :param key: The parameter key to test.
+        :type key: str
+        :param params: The request parameters.
+        :type params: dict
+        :param default: The default value if no key is passed.
+        :type default: bool or None
         """
         if key not in params:
             return default
 
-        val = params[key]
-
-        if isinstance(val, bool):
-            return val
-
-        return val.lower().strip() in ('true', 'on', '1', 'yes')
+        return toBool(params[key])
 
     def requireAdmin(self, user, message=None):
         """
@@ -925,8 +986,7 @@ class Resource(ModelImporter):
         """
         return setRawResponse(*args, **kwargs)
 
-    def getPagingParameters(self, params, defaultSortField=None,
-                            defaultSortDir=SortDir.ASCENDING):
+    def getPagingParameters(self, params, defaultSortField=None, defaultSortDir=SortDir.ASCENDING):
         """
         Pass the URL parameters into this function if the request is for a
         list of resources that should be paginated. It will return a tuple of
@@ -963,7 +1023,7 @@ class Resource(ModelImporter):
         designated scope or set of scopes. Raises an AccessException if not.
 
         :param scope: A scope or set of scopes that is required.
-        :type scope: str or list of str
+        :type scope: `str or list of str`
         """
         ensureTokenScopes(getCurrentToken(), scope)
 
@@ -973,6 +1033,12 @@ class Resource(ModelImporter):
         """
         return getBodyJson()
 
+    def getParamJson(self, name, params, default=None):
+        """
+        Bound wrapper for :func:`girder.api.rest.getParamJson`.
+        """
+        return getParamJson(name, params, default)
+
     def getCurrentToken(self):
         """
         Returns the current valid token object that was passed via the token
