@@ -19,13 +19,16 @@
 
 import cherrypy
 import errno
+import os
 import six
 
-from ..describe import Description, describeRoute
-from ..rest import Resource, RestException, filtermodel, loadmodel
+from ..describe import Description, autoDescribeRoute, describeRoute
+from ..rest import Resource, RestException, filtermodel
 from ...constants import AccessType, TokenScope
 from girder.models.model_base import AccessException, GirderException
 from girder.api import access
+from girder.utility import RequestBodyStream
+from girder.utility.progress import ProgressContext
 
 
 class File(Resource):
@@ -51,11 +54,10 @@ class File(Resource):
         self.route('PUT', (':id', 'move'), self.moveFileToAssetstore)
 
     @access.public(scope=TokenScope.DATA_READ)
-    @loadmodel(model='file', level=AccessType.READ)
     @filtermodel(model='file')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get a file\'s information.')
-        .param('id', 'The ID of the file.', paramType='path')
+        .modelParam('id', model='file', level=AccessType.READ)
         .errorResponse()
         .errorResponse('Read access was denied on the file.', 403)
     )
@@ -63,27 +65,27 @@ class File(Resource):
         return file
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Start a new upload or create an empty or link file.')
         .responseClass('Upload')
-        .param('parentType', 'Type being uploaded into (folder or item).')
+        .param('parentType', 'Type being uploaded into.', enum=['folder', 'item'])
         .param('parentId', 'The ID of the parent.')
         .param('name', 'Name of the file being created.')
-        .param('size', 'Size in bytes of the file.',
-               dataType='integer', required=False)
+        .param('size', 'Size in bytes of the file.', dataType='integer', required=False)
         .param('mimeType', 'The MIME type of the file.', required=False)
         .param('linkUrl', 'If this is a link file, pass its URL instead '
                'of size and mimeType using this parameter.', required=False)
         .param('reference', 'If included, this information is passed to the '
                'data.process event when the upload is complete.',
                required=False)
-        .param('assetstoreId', 'Direct the upload to a specific assetstore.',
+        .param('assetstoreId', 'Direct the upload to a specific assetstore (admin-only).',
                required=False)
         .errorResponse()
         .errorResponse('Write access was denied on the parent folder.', 403)
         .errorResponse('Failed to create upload.', 500)
     )
-    def initUpload(self, params):
+    def initUpload(self, parentType, parentId, name, size, mimeType, linkUrl, reference,
+                   assetstoreId, params):
         """
         Before any bytes of the actual file are sent, a request should be made
         to initialize the upload. This creates the temporary record of the
@@ -91,39 +93,30 @@ class File(Resource):
         method. If you pass a "linkUrl" parameter, it will make a link file
         in the designated parent.
         """
-        self.requireParams(('name', 'parentId', 'parentType'), params)
         user = self.getCurrentUser()
+        parent = self.model(parentType).load(
+            id=parentId, user=user, level=AccessType.WRITE, exc=True)
 
-        mimeType = params.get('mimeType', 'application/octet-stream')
-        parentType = params['parentType'].lower()
-
-        if parentType not in ('folder', 'item'):
-            raise RestException('The parentType must be "folder" or "item".')
-
-        parent = self.model(parentType).load(id=params['parentId'], user=user,
-                                             level=AccessType.WRITE, exc=True)
-
-        if 'linkUrl' in params:
+        if linkUrl is not None:
             return self.model('file').filter(
                 self.model('file').createLinkFile(
-                    url=params['linkUrl'], parent=parent, name=params['name'],
-                    parentType=parentType, creator=user), user)
+                    url=linkUrl, parent=parent, name=name, parentType=parentType, creator=user,
+                    size=size, mimeType=mimeType), user)
         else:
-            self.requireParams('size', params)
+            self.requireParams({'size': size})
             assetstore = None
-            if params.get('assetstoreId'):
-                assetstore = self.model('assetstore').load(
-                    params['assetstoreId'])
+            if assetstoreId:
+                self.requireAdmin(
+                    user, message='You must be an admin to select a destination assetstore.')
+                assetstore = self.model('assetstore').load(assetstoreId)
             try:
                 upload = self.model('upload').createUpload(
-                    user=user, name=params['name'], parentType=parentType,
-                    parent=parent, size=int(params['size']), mimeType=mimeType,
-                    reference=params.get('reference'), assetstore=assetstore)
+                    user=user, name=name, parentType=parentType, parent=parent, size=size,
+                    mimeType=mimeType, reference=reference, assetstore=assetstore)
             except OSError as exc:
                 if exc.errno == errno.EACCES:
                     raise GirderException(
-                        'Failed to create upload.',
-                        'girder.api.v1.file.create-upload-failed')
+                        'Failed to create upload.', 'girder.api.v1.file.create-upload-failed')
                 raise
             if upload['size'] > 0:
                 return upload
@@ -132,23 +125,20 @@ class File(Resource):
                     self.model('upload').finalizeUpload(upload), user)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Finalize an upload explicitly if necessary.')
         .notes('This is only required in certain non-standard upload '
                'behaviors. Clients should know which behavior models require '
                'the finalize step to be called in their behavior handlers.')
-        .param('uploadId', 'The ID of the upload record.', paramType='formData')
+        .modelParam('uploadId', paramType='formData')
         .errorResponse(('ID was invalid.',
                         'The upload does not require finalization.',
                         'Not enough bytes have been uploaded.'))
         .errorResponse('You are not the user who initiated the upload.', 403)
     )
-    def finalizeUpload(self, params):
-        self.requireParams('uploadId', params)
+    def finalizeUpload(self, upload, params):
         user = self.getCurrentUser()
 
-        upload = self.model('upload').load(params['uploadId'], exc=True)
-
         if upload['userId'] != user['_id']:
             raise AccessException('You did not initiate this upload.')
 
@@ -157,29 +147,26 @@ class File(Resource):
         # complete the upload.
         if upload['received'] != upload['size'] and 'behavior' not in upload:
             raise RestException(
-                'Server has only received %s bytes, but the file should be %s '
-                'bytes.' % (upload['received'], upload['size']))
+                'Server has only received %s bytes, but the file should be %s bytes.' %
+                (upload['received'], upload['size']))
 
         file = self.model('upload').finalizeUpload(upload)
         extraKeys = file.get('additionalFinalizeKeys', ())
         return self.model('file').filter(file, user, additionalKeys=extraKeys)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Request required offset before resuming an upload.')
-        .param('uploadId', 'The ID of the upload record.')
-        .errorResponse("The ID was invalid, or the offset did not match the "
-                       "server's record.")
+        .modelParam('uploadId', paramType='formData')
+        .errorResponse("The ID was invalid, or the offset did not match the server's record.")
     )
-    def requestOffset(self, params):
+    def requestOffset(self, upload, params):
         """
         This should be called when resuming an interrupted upload. It will
         report the offset into the upload that should be used to resume.
         :param uploadId: The _id of the temp upload record being resumed.
         :returns: The offset in bytes that the client should use.
         """
-        self.requireParams('uploadId', params)
-        upload = self.model('upload').load(params['uploadId'], exc=True)
         offset = self.model('upload').requestOffset(upload)
 
         if isinstance(offset, six.integer_types):
@@ -190,23 +177,18 @@ class File(Resource):
             return offset
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @describeRoute(
-        Description('Upload a chunk of a file with multipart/form-data.')
-        .consumes('multipart/form-data')
-        .param('uploadId', 'The ID of the upload record.', paramType='formData')
+    @autoDescribeRoute(
+        Description('Upload a chunk of a file.')
+        .modelParam('uploadId', paramType='formData')
         .param('offset', 'Offset of the chunk in the file.', dataType='integer',
                paramType='formData')
-        .param('chunk', 'The actual bytes of the chunk. For external upload '
-               'behaviors, this may be set to an opaque string that will be '
-               'handled by the assetstore adapter.',
-               dataType='file', paramType='formData')
         .errorResponse(('ID was invalid.',
                         'Received too many bytes.',
                         'Chunk is smaller than the minimum size.'))
         .errorResponse('You are not the user who initiated the upload.', 403)
         .errorResponse('Failed to store upload.', 500)
     )
-    def readChunk(self, params):
+    def readChunk(self, upload, offset, params):
         """
         After the temporary upload record has been created (see initUpload),
         the bytes themselves should be passed up in ordered chunks. The user
@@ -214,13 +196,25 @@ class File(Resource):
         the writer of the chunk is the same as the person who initiated the
         upload. The passed offset is a verification mechanism for ensuring the
         server and client agree on the number of bytes sent/received.
+
+        This method accepts both the legacy multipart content encoding, as
+        well as passing offset and uploadId as query parameters and passing
+        the chunk as the body, which is the recommended method.
+
+        Multipart uploads are @deprecated as of v2.2.0.
         """
-        self.requireParams(('offset', 'uploadId', 'chunk'), params)
+        if 'chunk' in params:
+            chunk = params['chunk']
+            if isinstance(chunk, cherrypy._cpreqbody.Part):
+                # Seek is the only obvious way to get the length of the part
+                chunk.file.seek(0, os.SEEK_END)
+                size = chunk.file.tell()
+                chunk.file.seek(0, os.SEEK_SET)
+                chunk = RequestBodyStream(chunk.file, size=size)
+        else:
+            chunk = RequestBodyStream(cherrypy.request.body)
 
         user = self.getCurrentUser()
-        upload = self.model('upload').load(params['uploadId'], exc=True)
-        offset = int(params['offset'])
-        chunk = params['chunk']
 
         if upload['userId'] != user['_id']:
             raise AccessException('You did not initiate this upload.')
@@ -230,10 +224,7 @@ class File(Resource):
                 'Server has received %s bytes, but client sent offset %s.' % (
                     upload['received'], offset))
         try:
-            if isinstance(chunk, cherrypy._cpreqbody.Part):
-                return self.model('upload').handleChunk(upload, chunk.file)
-            else:
-                return self.model('upload').handleChunk(upload, chunk)
+            return self.model('upload').handleChunk(upload, chunk, filter=True, user=user)
         except IOError as exc:
             if exc.errno == errno.EACCES:
                 raise Exception('Failed to store upload.')
@@ -241,14 +232,13 @@ class File(Resource):
 
     @access.cookie
     @access.public(scope=TokenScope.DATA_READ)
-    @loadmodel(model='file', level=AccessType.READ)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Download a file.')
         .notes('This endpoint also accepts the HTTP "Range" header for partial '
                'file downloads.')
-        .param('id', 'The ID of the file.', paramType='path')
+        .modelParam('id', model='file', level=AccessType.READ)
         .param('offset', 'Start downloading at this offset in bytes within '
-               'the file.', dataType='integer', required=False)
+               'the file.', dataType='integer', required=False, default=0)
         .param('endByte', 'If you only wish to download part of the file, '
                'pass this as the index of the last byte to download. Unlike '
                'the HTTP Range header, the endByte parameter is non-inclusive, '
@@ -256,14 +246,14 @@ class File(Resource):
                'final byte you wish to receive.', dataType='integer',
                required=False)
         .param('contentDisposition', 'Specify the Content-Disposition response '
-               'header disposition-type value', required=False,
+               'header disposition-type value.', required=False,
                enum=['inline', 'attachment'], default='attachment')
-        .param('extraParameters', 'Arbitrary data to send along with the '
-               'download request.', required=False)
+        .param('extraParameters', 'Arbitrary data to send along with the download request.',
+               required=False)
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied on the parent folder.', 403)
     )
-    def download(self, file, params):
+    def download(self, file, offset, endByte, contentDisposition, extraParameters, params):
         """
         Defers to the underlying assetstore adapter to stream a file out.
         Requires read permission on the folder that contains the file's item.
@@ -275,24 +265,10 @@ class File(Resource):
         if rangeHeader and len(rangeHeader):
             # Currently we only support a single range.
             offset, endByte = rangeHeader[0]
-        else:
-            offset = int(params.get('offset', 0))
-            endByte = params.get('endByte')
 
-            if endByte is not None:
-                endByte = int(endByte)
-
-        contentDisp = params.get('contentDisposition', None)
-        if (contentDisp is not None and
-           contentDisp not in {'inline', 'attachment'}):
-            raise RestException('Unallowed contentDisposition type "%s".' %
-                                contentDisp)
-
-        extraParameters = params.get('extraParameters')
-
-        return self.model('file').download(file, offset, endByte=endByte,
-                                           contentDisposition=contentDisp,
-                                           extraParameters=extraParameters)
+        return self.model('file').download(
+            file, offset, endByte=endByte, contentDisposition=contentDisposition,
+            extraParameters=extraParameters)
 
     @access.cookie
     @access.public(scope=TokenScope.DATA_READ)
@@ -313,10 +289,9 @@ class File(Resource):
         return self.download(id=id, params=params)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='file', level=AccessType.WRITE)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Delete a file by ID.')
-        .param('id', 'The ID of the file.', paramType='path')
+        .modelParam('id', model='file', level=AccessType.WRITE)
         .errorResponse('ID was invalid.')
         .errorResponse('Write access was denied on the parent folder.', 403)
     )
@@ -324,10 +299,9 @@ class File(Resource):
         self.model('file').remove(file)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='upload')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Cancel a partially completed upload.')
-        .param('id', 'The ID of the upload.', paramType='path')
+        .modelParam('id', model='upload')
         .errorResponse('ID was invalid.')
         .errorResponse('You lack permission to cancel this upload.', 403)
     )
@@ -341,77 +315,77 @@ class File(Resource):
         return {'message': 'Upload canceled.'}
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='file', level=AccessType.WRITE)
     @filtermodel(model='file')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Change file metadata such as name or MIME type.')
-        .param('id', 'The ID of the file.', paramType='path')
-        .param('name', 'The name to set on the file.', required=False)
-        .param('mimeType', 'The MIME type of the file.', required=False)
+        .modelParam('id', model='file', level=AccessType.WRITE)
+        .param('name', 'The name to set on the file.', required=False, strip=True)
+        .param('mimeType', 'The MIME type of the file.', required=False, strip=True)
         .errorResponse('ID was invalid.')
         .errorResponse('Write access was denied on the parent folder.', 403)
     )
-    def updateFile(self, file, params):
-        file['name'] = params.get('name', file['name']).strip()
-        file['mimeType'] = params.get('mimeType',
-                                      (file.get('mimeType') or '').strip())
+    def updateFile(self, file, name, mimeType, params):
+        if name is not None:
+            file['name'] = name
+        if mimeType is not None:
+            file['mimeType'] = mimeType
+
         return self.model('file').updateFile(file)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='file', level=AccessType.WRITE)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Change the contents of an existing file.')
-        .param('id', 'The ID of the file.', paramType='path')
+        .modelParam('id', model='file', level=AccessType.WRITE)
         .param('size', 'Size in bytes of the new file.', dataType='integer')
         .param('reference', 'If included, this information is passed to the '
                'data.process event when the upload is complete.',
                required=False)
-        .param('assetstoreId', 'Direct the upload to a specific assetstore.',
+        .param('assetstoreId', 'Direct the upload to a specific assetstore (admin-only).',
                required=False)
         .notes('After calling this, send the chunks just like you would with a '
                'normal file upload.')
     )
-    def updateFileContents(self, file, params):
-        self.requireParams('size', params)
+    def updateFileContents(self, file, size, reference, assetstoreId, params):
         user = self.getCurrentUser()
 
         assetstore = None
-        if params.get('assetstoreId'):
-            assetstore = self.model('assetstore').load(params['assetstoreId'])
+        if assetstoreId:
+            self.requireAdmin(
+                user, message='You must be an admin to select a destination assetstore.')
+            assetstore = self.model('assetstore').load(assetstoreId)
         # Create a new upload record into the existing file
         upload = self.model('upload').createUploadToFile(
-            file=file, user=user, size=int(params['size']),
-            reference=params.get('reference'), assetstore=assetstore)
+            file=file, user=user, size=size, reference=reference, assetstore=assetstore)
 
         if upload['size'] > 0:
             return upload
         else:
-            return self.model('file').filter(
-                self.model('upload').finalizeUpload(upload), user)
+            return self.model('file').filter(self.model('upload').finalizeUpload(upload), user)
 
-    @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='file', level=AccessType.WRITE)
-    @describeRoute(
-        Description('Move a file to a different assetstore.')
-        .param('id', 'The ID of the file.', paramType='path')
-        .param('assetstoreId', 'The destination assetstore.')
-    )
-    def moveFileToAssetstore(self, file, params):
-        self.requireParams('assetstoreId', params)
-        user = self.getCurrentUser()
-        assetstore = self.model('assetstore').load(params['assetstoreId'])
-        return self.model('upload').moveFileToAssetstore(
-            file=file, user=user, assetstore=assetstore)
-
-    @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='file', level=AccessType.READ)
-    @loadmodel(model='item', map={'itemId': 'item'}, level=AccessType.WRITE)
+    @access.admin(scope=TokenScope.DATA_WRITE)
     @filtermodel(model='file')
-    @describeRoute(
+    @autoDescribeRoute(
+        Description('Move a file to a different assetstore.')
+        .modelParam('id', model='file', level=AccessType.WRITE)
+        .modelParam('assetstoreId', 'The destination assetstore.', paramType='formData')
+        .param('progress', 'Controls whether progress notifications will be sent.',
+               dataType='boolean', default=False, required=False)
+    )
+    def moveFileToAssetstore(self, file, assetstore, progress, params):
+        user = self.getCurrentUser()
+        title = 'Moving file "%s" to assetstore "%s"' % (file['name'], assetstore['name'])
+
+        with ProgressContext(progress, user=user, title=title, total=file['size']) as ctx:
+            return self.model('upload').moveFileToAssetstore(
+                file=file, user=user, assetstore=assetstore, progress=ctx)
+
+    @access.user(scope=TokenScope.DATA_WRITE)
+    @filtermodel(model='file')
+    @autoDescribeRoute(
         Description('Copy a file.')
-        .param('id', 'The ID of the file.', paramType='path')
-        .param('itemId', 'The item to copy the file to.', required=True)
+        .modelParam('id', model='file', level=AccessType.READ)
+        .modelParam('itemId', description='The ID of the item to copy the file to.',
+                    level=AccessType.WRITE, paramType='formData')
     )
     def copy(self, file, item, params):
-        return self.model('file').copyFile(
-            file, self.getCurrentUser(), item=item)
+        return self.model('file').copyFile(file, self.getCurrentUser(), item=item)
