@@ -17,11 +17,8 @@
 #  limitations under the License.
 ###############################################################################
 
-import json
-
-from ..describe import Description, describeRoute
-from ..rest import Resource, RestException, filtermodel, loadmodel, \
-    setResponseHeader
+from ..describe import Description, autoDescribeRoute
+from ..rest import Resource, RestException, filtermodel, setResponseHeader
 from girder.api import access
 from girder.constants import AccessType, TokenScope
 from girder.utility import ziputil
@@ -46,10 +43,11 @@ class Folder(Resource):
         self.route('PUT', (':id', 'access'), self.updateFolderAccess)
         self.route('POST', (':id', 'copy'), self.copyFolder)
         self.route('PUT', (':id', 'metadata'), self.setMetadata)
+        self.route('DELETE', (':id', 'metadata'), self.deleteMetadata)
 
     @access.public(scope=TokenScope.DATA_READ)
     @filtermodel(model='folder')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Search for folders by certain properties.')
         .responseClass('Folder', array=True)
         .param('parentType', "Type of the folder's parent", required=False,
@@ -57,13 +55,12 @@ class Folder(Resource):
         .param('parentId', "The ID of the folder's parent.", required=False)
         .param('text', 'Pass to perform a text search.', required=False)
         .param('name', 'Pass to lookup a folder by exact name match. Must '
-               'pass parentType and parentId as well when using this.',
-               required=False)
+               'pass parentType and parentId as well when using this.', required=False)
         .pagingParams(defaultSort='lowerName')
         .errorResponse()
         .errorResponse('Read access was denied on the parent resource.', 403)
     )
-    def find(self, params):
+    def find(self, parentType, parentId, text, name, limit, offset, sort, params):
         """
         Get a list of folders with given search parameters. Currently accepted
         search modes are:
@@ -75,42 +72,33 @@ class Folder(Resource):
         2. Searching with full text search across all folders in the system.
            Simply pass a "text" parameter for this mode.
         """
-        limit, offset, sort = self.getPagingParameters(params, 'lowerName')
         user = self.getCurrentUser()
 
-        if 'parentId' in params and 'parentType' in params:
-            parentType = params['parentType'].lower()
-            if parentType not in ('collection', 'folder', 'user'):
-                raise RestException('The parentType must be user, collection,'
-                                    ' or folder.')
-
+        if parentType and parentId:
             parent = self.model(parentType).load(
-                id=params['parentId'], user=user, level=AccessType.READ,
-                exc=True)
+                parentId, user=user, level=AccessType.READ, exc=True)
 
             filters = {}
-            if params.get('text'):
+            if text:
                 filters['$text'] = {
-                    '$search': params['text']
+                    '$search': text
                 }
-            if params.get('name'):
-                filters['name'] = params['name']
+            if name:
+                filters['name'] = name
 
             return list(self.model('folder').childFolders(
                 parentType=parentType, parent=parent, user=user,
                 offset=offset, limit=limit, sort=sort, filters=filters))
-        elif 'text' in params:
+        elif text:
             return list(self.model('folder').textSearch(
-                params['text'], user=user, limit=limit, offset=offset,
-                sort=sort))
+                text, user=user, limit=limit, offset=offset, sort=sort))
         else:
             raise RestException('Invalid search mode.')
 
     @access.public(scope=TokenScope.DATA_READ)
-    @loadmodel(model='folder', level=AccessType.READ)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get detailed information about a folder.')
-        .param('id', 'The ID of the folder.', paramType='path')
+        .modelParam('id', model='folder', level=AccessType.READ)
         .errorResponse()
         .errorResponse('Read access was denied on the folder.', 403)
     )
@@ -123,34 +111,24 @@ class Folder(Resource):
 
     @access.cookie
     @access.public(scope=TokenScope.DATA_READ)
-    @loadmodel(model='folder', level=AccessType.READ)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Download an entire folder as a zip archive.')
-        .param('id', 'The ID of the folder.', paramType='path')
-        .param('mimeFilter', 'JSON list of MIME types to include.',
-               required=False)
+        .modelParam('id', model='folder', level=AccessType.READ)
+        .jsonParam('mimeFilter', 'JSON list of MIME types to include.', required=False,
+                   requireArray=True)
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied for the folder.', 403)
     )
-    def downloadFolder(self, folder, params):
+    def downloadFolder(self, folder, mimeFilter, params):
         """
         Returns a generator function that will be used to stream out a zip
         file containing this folder's contents, filtered by permissions.
         """
         setResponseHeader('Content-Type', 'application/zip')
         setResponseHeader(
-            'Content-Disposition',
-            'attachment; filename="%s%s"' % (folder['name'], '.zip'))
+            'Content-Disposition', 'attachment; filename="%s%s"' % (folder['name'], '.zip'))
 
         user = self.getCurrentUser()
-        mimeFilter = params.get('mimeFilter')
-        if mimeFilter:
-            try:
-                mimeFilter = json.loads(mimeFilter)
-                if not isinstance(mimeFilter, list):
-                    raise ValueError()
-            except ValueError:
-                raise RestException('The mimeFilter must be a JSON list.')
 
         def stream():
             zip = ziputil.ZipGenerator(folder['name'])
@@ -162,51 +140,45 @@ class Folder(Resource):
         return stream
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='folder', level=AccessType.WRITE)
     @filtermodel(model='folder')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Update a folder or move it into a new parent.')
         .responseClass('Folder')
-        .param('id', 'The ID of the folder.', paramType='path')
-        .param('name', 'Name of the folder.', required=False)
-        .param('description', 'Description for the folder.', required=False)
+        .modelParam('id', model='folder', level=AccessType.WRITE)
+        .param('name', 'Name of the folder.', required=False, strip=True)
+        .param('description', 'Description for the folder.', required=False, strip=True)
         .param('parentType', "Type of the folder's parent", required=False,
-               enum=['folder', 'user', 'collection'])
-        .param('parentId', 'Parent ID for the new parent of this folder.',
-               required=False)
+               enum=['folder', 'user', 'collection'], strip=True)
+        .param('parentId', 'Parent ID for the new parent of this folder.', required=False)
         .errorResponse('ID was invalid.')
-        .errorResponse('Write access was denied for the folder or its new '
-                       'parent object.', 403)
+        .errorResponse('Write access was denied for the folder or its new parent object.', 403)
     )
-    def updateFolder(self, folder, params):
+    def updateFolder(self, folder, name, description, parentType, parentId, params):
         user = self.getCurrentUser()
-        folder['name'] = params.get('name', folder['name']).strip()
-        folder['description'] = params.get(
-            'description', folder['description']).strip()
+        if name is not None:
+            folder['name'] = name
+        if description is not None:
+            folder['description'] = description
 
         folder = self.model('folder').updateFolder(folder)
 
-        if 'parentType' in params and 'parentId' in params:
-            parentType = params['parentType'].lower()
-            if parentType not in ('user', 'collection', 'folder'):
-                raise RestException('Invalid parentType.')
-
+        if parentType and parentId:
             parent = self.model(parentType).load(
-                params['parentId'], level=AccessType.WRITE, user=user, exc=True)
-            if (parentType, parent['_id']) !=\
-               (folder['parentCollection'], folder['parentId']):
+                parentId, level=AccessType.WRITE, user=user, exc=True)
+            if (parentType, parent['_id']) != (folder['parentCollection'], folder['parentId']):
                 folder = self.model('folder').move(folder, parent, parentType)
 
         return folder
 
     @access.user(scope=TokenScope.DATA_OWN)
-    @loadmodel(model='folder', level=AccessType.ADMIN)
     @filtermodel(model='folder', addFields={'access'})
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Update the access control list for a folder.')
-        .param('id', 'The ID of the folder.', paramType='path')
-        .param('access', 'The JSON-encoded access control list.')
-        .param('public', "Whether the folder should be publicly visible.",
+        .modelParam('id', model='folder', level=AccessType.ADMIN)
+        .jsonParam('access', 'The JSON-encoded access control list.', requireObject=True)
+        .jsonParam('publicFlags', 'JSON list of public access flags.', requireArray=True,
+                   required=False)
+        .param('public', 'Whether the folder should be publicly visible.',
                dataType='boolean', required=False)
         .param('recurse', 'Whether the policies should be applied to all '
                'subfolders under this folder as well.', dataType='boolean',
@@ -217,19 +189,9 @@ class Folder(Resource):
         .errorResponse('ID was invalid.')
         .errorResponse('Admin access was denied for the folder.', 403)
     )
-    def updateFolderAccess(self, folder, params):
-        self.requireParams('access', params)
+    def updateFolderAccess(self, folder, access, publicFlags, public, recurse, progress, params):
         user = self.getCurrentUser()
-
-        public = self.boolParam('public', params)
-        recurse = self.boolParam('recurse', params, default=False)
-        progress = self.boolParam('progress', params, default=False) and recurse
-
-        try:
-            access = json.loads(params['access'])
-        except ValueError:
-            raise RestException('The access parameter must be JSON.')
-
+        progress = progress and recurse  # Only enable progress in recursive case
         with ProgressContext(progress, user=user, title='Updating permissions',
                              message='Calculating progress...') as ctx:
             if progress:
@@ -238,18 +200,19 @@ class Folder(Resource):
                     level=AccessType.ADMIN))
             return self.model('folder').setAccessList(
                 folder, access, save=True, recurse=recurse, user=user,
-                progress=ctx, setPublic=public)
+                progress=ctx, setPublic=public, publicFlags=publicFlags)
 
     @access.user(scope=TokenScope.DATA_WRITE)
     @filtermodel(model='folder')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Create a new folder.')
         .responseClass('Folder')
         .param('parentType', "Type of the folder's parent", required=False,
-               enum=['folder', 'user', 'collection'])
+               enum=['folder', 'user', 'collection'], default='folder')
         .param('parentId', "The ID of the folder's parent.")
-        .param('name', "Name of the folder.")
-        .param('description', "Description for the folder.", required=False)
+        .param('name', 'Name of the folder.', strip=True)
+        .param('description', 'Description for the folder.', required=False,
+               default='', strip=True)
         .param('public', "Whether the folder should be publicly visible. By "
                "default, inherits the value from parent folder, or in the "
                "case of user or collection parentType, defaults to False.",
@@ -257,47 +220,21 @@ class Folder(Resource):
         .errorResponse()
         .errorResponse('Write access was denied on the parent', 403)
     )
-    def createFolder(self, params):
-        """
-        Create a new folder.
-
-        :param parentId: The _id of the parent folder.
-        :type parentId: str
-        :param parentType: The type of the parent of this folder.
-        :type parentType: str - 'user', 'collection', or 'folder'
-        :param name: The name of the folder to create.
-        :param description: Folder description.
-        :param public: Public read access flag.
-        :type public: bool
-        """
-        self.requireParams(('name', 'parentId'), params)
-
+    def createFolder(self, public, parentType, parentId, name, description, params):
         user = self.getCurrentUser()
-        parentType = params.get('parentType', 'folder').lower()
-        name = params['name'].strip()
-        description = params.get('description', '').strip()
-        public = self.boolParam('public', params, default=None)
-
-        if parentType not in ('folder', 'user', 'collection'):
-            raise RestException('Set parentType to collection, folder, '
-                                'or user.')
-
-        model = self.model(parentType)
-
-        parent = model.load(id=params['parentId'], user=user,
-                            level=AccessType.WRITE, exc=True)
+        parent = self.model(parentType).load(
+            id=parentId, user=user, level=AccessType.WRITE, exc=True)
 
         return self.model('folder').createFolder(
             parent=parent, name=name, parentType=parentType, creator=user,
             description=description, public=public)
 
     @access.public(scope=TokenScope.DATA_READ)
-    @loadmodel(model='folder', level=AccessType.READ)
     @filtermodel(model='folder')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get a folder by ID.')
         .responseClass('Folder')
-        .param('id', 'The ID of the folder.', paramType='path')
+        .modelParam('id', model='folder', level=AccessType.READ)
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied for the folder.', 403)
     )
@@ -305,11 +242,10 @@ class Folder(Resource):
         return folder
 
     @access.user(scope=TokenScope.DATA_OWN)
-    @loadmodel(model='folder', level=AccessType.ADMIN)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get the access control list for a folder.')
         .responseClass('Folder')
-        .param('id', 'The ID of the folder.', paramType='path')
+        .modelParam('id', model='folder', level=AccessType.ADMIN)
         .errorResponse('ID was invalid.')
         .errorResponse('Admin access was denied for the folder.', 403)
     )
@@ -317,17 +253,15 @@ class Folder(Resource):
         return self.model('folder').getFullAccessList(folder)
 
     @access.user(scope=TokenScope.DATA_OWN)
-    @loadmodel(model='folder', level=AccessType.ADMIN)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Delete a folder by ID.')
-        .param('id', 'The ID of the folder.', paramType='path')
-        .param('progress', 'Whether to record progress on this task. Default '
-               'is false.', required=False, dataType='boolean')
+        .modelParam('id', model='folder', level=AccessType.ADMIN)
+        .param('progress', 'Whether to record progress on this task.',
+               required=False, dataType='boolean', default=False)
         .errorResponse('ID was invalid.')
         .errorResponse('Admin access was denied for the folder.', 403)
     )
-    def deleteFolder(self, folder, params):
-        progress = self.boolParam('progress', params, default=False)
+    def deleteFolder(self, folder, progress, params):
         with ProgressContext(progress, user=self.getCurrentUser(),
                              title='Deleting folder %s' % folder['name'],
                              message='Calculating folder size...') as ctx:
@@ -338,38 +272,30 @@ class Folder(Resource):
         return {'message': 'Deleted folder %s.' % folder['name']}
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='folder', level=AccessType.WRITE)
     @filtermodel(model='folder')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Set metadata fields on an folder.')
         .responseClass('Folder')
         .notes('Set metadata fields to null in order to delete them.')
-        .param('id', 'The ID of the folder.', paramType='path')
-        .param('body', 'A JSON object containing the metadata keys to add',
-               paramType='body')
+        .modelParam('id', model='folder', level=AccessType.WRITE)
+        .jsonParam('metadata', 'A JSON object containing the metadata keys to add',
+                   paramType='body', requireObject=True)
+        .param('allowNull', 'Whether "null" is allowed as a metadata value.', required=False,
+               dataType='boolean', default=False)
         .errorResponse(('ID was invalid.',
                         'Invalid JSON passed in request body.',
                         'Metadata key name was invalid.'))
         .errorResponse('Write access was denied for the folder.', 403)
     )
-    def setMetadata(self, folder, params):
-        metadata = self.getBodyJson()
-
-        # Make sure we let user know if we can't accept a metadata key
-        for k in metadata:
-            if '.' in k or k[0] == '$':
-                raise RestException('The key name %s must not contain a '
-                                    'period or begin with a dollar sign.' % k)
-
-        return self.model('folder').setMetadata(folder, metadata)
+    def setMetadata(self, folder, metadata, allowNull, params):
+        return self.model('folder').setMetadata(folder, metadata, allowNull=allowNull)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='folder', level=AccessType.READ)
     @filtermodel(model='folder')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Copy a folder.')
         .responseClass('Folder')
-        .param('id', 'The ID of the original folder.', paramType='path')
+        .modelParam('id', 'The ID of the original folder.', model='folder', level=AccessType.READ)
         .param('parentType', "Type of the new folder's parent", required=False,
                enum=['folder', 'user', 'collection'])
         .param('parentId', 'The ID of the parent document.', required=False)
@@ -380,26 +306,22 @@ class Folder(Resource):
                "of user or collection parentType, defaults to False. If "
                "'original', use the value of the original folder.",
                required=False, enum=[True, False, 'original'])
-        .param('progress', 'Whether to record progress on this task. Default '
-               'is false.', required=False, dataType='boolean')
+        .param('progress', 'Whether to record progress on this task.',
+               required=False, dataType='boolean', default=False)
         .errorResponse(('A parameter was invalid.',
                         'ID was invalid.'))
         .errorResponse('Read access was denied on the original folder.\n\n'
                        'Write access was denied on the parent.', 403)
     )
-    def copyFolder(self, folder, params):
+    def copyFolder(self, folder, parentType, parentId, name, description, public, progress, params):
         user = self.getCurrentUser()
-        parentType = params.get('parentType', folder['parentCollection'])
-        if 'parentId' in params:
-            parentId = params.get('parentId', folder['parentId'])
+        parentType = parentType or folder['parentCollection']
+        if parentId:
             parent = self.model(parentType).load(
                 id=parentId, user=user, level=AccessType.WRITE, exc=True)
         else:
             parent = None
-        name = params.get('name', None)
-        description = params.get('description', None)
-        public = params.get('public', None)
-        progress = self.boolParam('progress', params, default=False)
+
         with ProgressContext(progress, user=self.getCurrentUser(),
                              title='Copying folder %s' % folder['name'],
                              message='Calculating folder size...') as ctx:
@@ -408,23 +330,20 @@ class Folder(Resource):
                 ctx.update(total=self.model('folder').subtreeCount(folder))
             return self.model('folder').copyFolder(
                 folder, creator=user, name=name, parentType=parentType,
-                parent=parent, description=description, public=public,
-                progress=ctx)
+                parent=parent, description=description, public=public, progress=ctx)
 
     @access.user(scope=TokenScope.DATA_WRITE)
-    @loadmodel(model='folder', level=AccessType.WRITE)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Remove all contents from a folder.')
         .notes('Cleans out all the items and subfolders from under a folder, '
                'but does not remove the folder itself.')
-        .param('id', 'The ID of the folder to clean.', paramType='path')
-        .param('progress', 'Whether to record progress on this task. Default '
-               'is false.', required=False, dataType='boolean', default=False)
+        .modelParam('id', 'The ID of the folder to clean.', model='folder', level=AccessType.WRITE)
+        .param('progress', 'Whether to record progress on this task.',
+               required=False, dataType='boolean', default=False)
         .errorResponse('ID was invalid.')
         .errorResponse('Write access was denied on the folder.', 403)
     )
-    def deleteContents(self, folder, params):
-        progress = self.boolParam('progress', params, default=False)
+    def deleteContents(self, folder, progress, params):
         with ProgressContext(progress, user=self.getCurrentUser(),
                              title='Clearing folder %s' % folder['name'],
                              message='Calculating folder size...') as ctx:
@@ -433,3 +352,26 @@ class Folder(Resource):
                 ctx.update(total=self.model('folder').subtreeCount(folder) - 1)
             self.model('folder').clean(folder, progress=ctx)
         return {'message': 'Cleaned folder %s.' % folder['name']}
+
+    @access.user(scope=TokenScope.DATA_WRITE)
+    @filtermodel('folder')
+    @autoDescribeRoute(
+        Description('Delete metadata fields on a folder.')
+        .responseClass('Folder')
+        .modelParam('id', model='folder', level=AccessType.WRITE)
+        .jsonParam(
+            'fields', 'A JSON list containing the metadata fields to delete',
+            paramType='body', schema={
+                'type': 'array',
+                'items': {
+                    'type': 'string'
+                }
+            }
+        )
+        .errorResponse(('ID was invalid.',
+                        'Invalid JSON passed in request body.',
+                        'Metadata key name was invalid.'))
+        .errorResponse('Write access was denied for the folder.', 403)
+    )
+    def deleteMetadata(self, folder, fields, params):
+        return self.model('folder').deleteMetadata(folder, fields)
