@@ -20,9 +20,11 @@
 import cherrypy
 import json
 import time
+from datetime import datetime
 
-from ..describe import Description, describeRoute
+from ..describe import Description, autoDescribeRoute
 from ..rest import Resource, setResponseHeader
+from girder.utility import JsonEncoder
 from girder.api import access
 
 # If no timeout param is passed to stream, we default to this value
@@ -37,7 +39,11 @@ def sseMessage(event):
     """
     Serializes an event into the server-sent events protocol.
     """
-    return 'data: %s\n\n' % json.dumps(event, default=str)
+    # Inject the current time on the server into the event so that
+    # the client doesn't need to worry about clock synchronization
+    # issues when restarting the event stream.
+    event['_girderTime'] = int(time.time())
+    return 'data: %s\n\n' % json.dumps(event, sort_keys=True, allow_nan=False, cls=JsonEncoder)
 
 
 class Notification(Resource):
@@ -48,9 +54,8 @@ class Notification(Resource):
 
     @access.cookie
     @access.token
-    @describeRoute(
-        Description('Stream notifications for a given user via the SSE '
-                    'protocol.')
+    @autoDescribeRoute(
+        Description('Stream notifications for a given user via the SSE protocol.')
         .notes('This uses long-polling to keep the connection open for '
                'several minutes at a time (or longer) and should be requested '
                'with an EventSource object or other SSE-capable client. '
@@ -58,31 +63,24 @@ class Notification(Resource):
                'they occur.  When no notification occurs for the timeout '
                'duration, the stream is closed. '
                '<p>This connection can stay open indefinitely long.')
-        .param('timeout', 'The duration without a notification before the '
-               'stream is closed.', dataType='integer', required=False)
+        .param('timeout', 'The duration without a notification before the stream is closed.',
+               dataType='integer', required=False, default=DEFAULT_STREAM_TIMEOUT)
+        .param('since', 'Filter out events before this time stamp.',
+               dataType='integer', required=False)
         .errorResponse()
         .errorResponse('You are not logged in.', 403)
     )
-    def stream(self, params):
-        """
-        Streams notifications using the server-sent events protocol. Closes
-        the connection if more than timeout seconds elapse without any new
-        notifications.
-
-        :params timeout: Timeout in seconds; if no notifications appear in
-            this duration, the connection will be closed. (default=300)
-        :type timeout: int
-        """
-        user = self.getCurrentUser()
-        token = self.getCurrentToken()
+    def stream(self, timeout, params):
+        user, token = self.getCurrentUser(returnToken=True)
 
         setResponseHeader('Content-Type', 'text/event-stream')
         setResponseHeader('Cache-Control', 'no-cache')
-
-        timeout = int(params.get('timeout', DEFAULT_STREAM_TIMEOUT))
+        since = params.get('since')
+        if since is not None:
+            since = datetime.utcfromtimestamp(since)
 
         def streamGen():
-            lastUpdate = None
+            lastUpdate = since
             start = time.time()
             wait = MIN_POLL_INTERVAL
             while cherrypy.engine.state == cherrypy.engine.states.STARTED:
