@@ -21,9 +21,9 @@ import base64
 import cherrypy
 import datetime
 
-from ..describe import Description, describeRoute
+from ..describe import Description, autoDescribeRoute
 from girder.api import access
-from girder.api.rest import Resource, RestException, AccessException, filtermodel, loadmodel
+from girder.api.rest import Resource, RestException, AccessException, filtermodel, setCurrentUser
 from girder.constants import AccessType, SettingKey, TokenScope
 from girder.models.token import genToken
 from girder.utility import mail_utils
@@ -57,35 +57,31 @@ class User(Resource):
 
     @access.public
     @filtermodel(model='user')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('List or search for users.')
         .responseClass('User', array=True)
-        .param('text', "Pass this to perform a full text search for items.",
-               required=False)
+        .param('text', "Pass this to perform a full text search for items.", required=False)
         .pagingParams(defaultSort='lastName')
     )
-    def find(self, params):
-        limit, offset, sort = self.getPagingParameters(params, 'lastName')
+    def find(self, text, limit, offset, sort, params):
         return list(self.model('user').search(
-            text=params.get('text'), user=self.getCurrentUser(), offset=offset,
-            limit=limit, sort=sort))
+            text=text, user=self.getCurrentUser(), offset=offset, limit=limit, sort=sort))
 
     @access.public(scope=TokenScope.USER_INFO_READ)
-    @loadmodel(map={'id': 'userToGet'}, model='user', level=AccessType.READ)
     @filtermodel(model='user')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get a user by ID.')
         .responseClass('User')
-        .param('id', 'The ID of the user.', paramType='path')
+        .modelParam('id', model='user', level=AccessType.READ)
         .errorResponse('ID was invalid.')
         .errorResponse('You do not have permission to see this user.', 403)
     )
-    def getUser(self, userToGet, params):
-        return userToGet
+    def getUser(self, user, params):
+        return user
 
     @access.public(scope=TokenScope.USER_INFO_READ)
     @filtermodel(model='user')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Retrieve the currently logged-in user information.')
         .responseClass('User')
     )
@@ -93,7 +89,7 @@ class User(Resource):
         return self.getCurrentUser()
 
     @access.public
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Log in to the system.')
         .notes('Pass your username and password using HTTP Basic Auth. Sends'
                ' a cookie that should be passed back in future requests.')
@@ -101,15 +97,9 @@ class User(Resource):
         .errorResponse('Invalid login or password.', 403)
     )
     def login(self, params):
-        """
-        Login endpoint. Sends an auth cookie in the response on success.
-        The caller is expected to use HTTP Basic Authentication when calling
-        this endpoint.
-        """
         user, token = self.getCurrentUser(returnToken=True)
 
-        # Only create and send new cookie if user isn't already sending
-        # a valid one.
+        # Only create and send new cookie if user isn't already sending a valid one.
         if not user:
             authHeader = cherrypy.request.headers.get('Girder-Authorization')
 
@@ -129,7 +119,7 @@ class User(Resource):
             login, password = credentials.split(':', 1)
             user = self.model('user').authenticate(login, password)
 
-            setattr(cherrypy.request, 'girderUser', user)
+            setCurrentUser(user)
             token = self.sendAuthTokenCookie(user)
 
         return {
@@ -143,7 +133,7 @@ class User(Resource):
         }
 
     @access.user
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Log out of the system.')
         .responseClass('Token')
         .notes('Attempts to delete your authentication cookie.')
@@ -157,7 +147,7 @@ class User(Resource):
 
     @access.public
     @filtermodel(model='user', addFields={'authToken'})
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Create a new user.')
         .responseClass('User')
         .param('login', "The user's requested login.")
@@ -166,21 +156,16 @@ class User(Resource):
         .param('lastName', "The user's last name.")
         .param('password', "The user's requested password")
         .param('admin', 'Whether this user should be a site administrator.',
-               required=False, dataType='boolean')
+               required=False, dataType='boolean', default=False)
         .errorResponse('A parameter was invalid, or the specified login or'
                        ' email already exists in the system.')
     )
-    def createUser(self, params):
-        self.requireParams(
-            ('firstName', 'lastName', 'login', 'password', 'email'), params)
-
+    def createUser(self, login, email, firstName, lastName, password, admin, params):
         currentUser = self.getCurrentUser()
 
         regPolicy = self.model('setting').get(SettingKey.REGISTRATION_POLICY)
 
-        if currentUser is not None and currentUser['admin']:
-            admin = self.boolParam('admin', params, default=False)
-        else:
+        if not currentUser or not currentUser['admin']:
             admin = False
             if regPolicy == 'closed':
                 raise RestException(
@@ -188,94 +173,83 @@ class User(Resource):
                     'administrator to create an account for you.')
 
         user = self.model('user').createUser(
-            login=params['login'], password=params['password'],
-            email=params['email'], firstName=params['firstName'],
-            lastName=params['lastName'], admin=admin)
+            login=login, password=password, email=email, firstName=firstName,
+            lastName=lastName, admin=admin)
 
-        outputUser = self.model('user').filter(user, user)
         if not currentUser and self.model('user').canLogin(user):
-            setattr(cherrypy.request, 'girderUser', user)
+            setCurrentUser(user)
             token = self.sendAuthTokenCookie(user)
-            outputUser['authToken'] = {
+            user['authToken'] = {
                 'token': token['_id'],
                 'expires': token['expires']
             }
-        return outputUser
+        return user
 
     @access.user
-    @loadmodel(map={'id': 'userToDelete'}, model='user', level=AccessType.ADMIN)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Delete a user by ID.')
-        .param('id', 'The ID of the user.', paramType='path')
+        .modelParam('id', model='user', level=AccessType.ADMIN)
         .errorResponse('ID was invalid.')
         .errorResponse('You do not have permission to delete this user.', 403)
     )
-    def deleteUser(self, userToDelete, params):
-        self.model('user').remove(userToDelete)
-        return {'message': 'Deleted user %s.' % userToDelete['login']}
+    def deleteUser(self, user, params):
+        self.model('user').remove(user)
+        return {'message': 'Deleted user %s.' % user['login']}
 
     @access.user
-    @loadmodel(model='user', level=AccessType.WRITE)
     @filtermodel(model='user')
-    @describeRoute(
+    @autoDescribeRoute(
         Description("Update a user's information.")
-        .param('id', 'The ID of the user.', paramType='path')
+        .modelParam('id', model='user', level=AccessType.WRITE)
         .param('firstName', 'First name of the user.')
         .param('lastName', 'Last name of the user.')
         .param('email', 'The email of the user.')
         .param('admin', 'Is the user a site admin (admin access required)',
                required=False, dataType='boolean')
         .param('status', 'The account status (admin access required)',
-               required=False, enum=['pending', 'enabled', 'disabled'])
+               required=False, enum=('pending', 'enabled', 'disabled'))
         .errorResponse()
         .errorResponse(('You do not have write access for this user.',
                         'Must be an admin to create an admin.'), 403)
     )
-    def updateUser(self, user, params):
-        self.requireParams(('firstName', 'lastName', 'email'), params)
-
-        user['firstName'] = params['firstName']
-        user['lastName'] = params['lastName']
-        user['email'] = params['email']
+    def updateUser(self, user, firstName, lastName, email, admin, status, params):
+        user['firstName'] = firstName
+        user['lastName'] = lastName
+        user['email'] = email
 
         # Only admins can change admin state
-        if 'admin' in params:
-            newAdminState = self.boolParam('admin', params)
+        if admin is True:
             if self.getCurrentUser()['admin']:
-                user['admin'] = newAdminState
-            else:
-                if newAdminState != user['admin']:
-                    raise AccessException('Only admins may change admin state.')
+                user['admin'] = admin
+            elif not user['admin']:
+                raise AccessException('Only admins may enable admin state.')
 
         # Only admins can change status
-        if 'status' in params and params['status'] != user.get('status', 'enabled'):
+        if status is not None and status != user.get('status', 'enabled'):
             if not self.getCurrentUser()['admin']:
                 raise AccessException('Only admins may change status.')
-            if user['status'] == 'pending' and params['status'] == 'enabled':
+            if user['status'] == 'pending' and status == 'enabled':
                 # Send email on the 'pending' -> 'enabled' transition
                 self.model('user')._sendApprovedEmail(user)
-            user['status'] = params['status']
+            user['status'] = status
 
-        user = self.model('user').save(user)
-        return self.model('user').filter(user, user)
+        return self.model('user').save(user)
 
     @access.admin
-    @loadmodel(model='user', level=AccessType.ADMIN)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Change a user\'s password.')
         .notes('Only administrators may use this endpoint.')
-        .param('id', 'The ID of the user.', paramType='path')
+        .modelParam('id', model='user', level=AccessType.ADMIN)
         .param('password', 'The user\'s new password.')
         .errorResponse('You are not an administrator.', 403)
         .errorResponse('The new password is invalid.')
     )
-    def changeUserPassword(self, user, params):
-        self.requireParams('password', params)
-        self.model('user').setPassword(user, params['password'])
+    def changeUserPassword(self, user, password, params):
+        self.model('user').setPassword(user, password)
         return {'message': 'Password changed.'}
 
     @access.user
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Change your password.')
         .param('old', 'Your current password or a temporary access token.')
         .param('new', 'Your new password.')
@@ -283,26 +257,24 @@ class User(Resource):
                         'Your old password is incorrect.'), 401)
         .errorResponse('Your new password is invalid.')
     )
-    def changePassword(self, params):
-        self.requireParams(('old', 'new'), params)
+    def changePassword(self, old, new, params):
         user = self.getCurrentUser()
         token = None
 
-        if not params['old']:
+        if not old:
             raise RestException('Old password must not be empty.')
 
         if (not self.model('password').hasPassword(user) or
-                not self.model('password').authenticate(user, params['old'])):
+                not self.model('password').authenticate(user, old)):
             # If not the user's actual password, check for temp access token
-            token = self.model('token').load(
-                params['old'], force=True, objectId=False, exc=False)
+            token = self.model('token').load(old, force=True, objectId=False, exc=False)
             if (not token or not token.get('userId') or
                     token['userId'] != user['_id'] or
                     not self.model('token').hasScope(
                         token, TokenScope.TEMPORARY_USER_AUTH)):
                 raise AccessException('Old password is incorrect.')
 
-        self.model('user').setPassword(user, params['new'])
+        self.model('user').setPassword(user, new)
 
         if token:
             # Remove the temporary access token if one was used
@@ -311,16 +283,13 @@ class User(Resource):
         return {'message': 'Password changed.'}
 
     @access.public
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Reset a forgotten password via email.')
-        .param('email', 'Your email address.')
+        .param('email', 'Your email address.', strip=True)
         .errorResponse('That email does not exist in the system.')
     )
-    def resetPassword(self, params):
-        self.requireParams('email', params)
-        email = params['email'].lower().strip()
-
-        user = self.model('user').findOne({'email': email})
+    def resetPassword(self, email, params):
+        user = self.model('user').findOne({'email': email.lower()})
         if user is None:
             raise RestException('That email is not registered.')
 
@@ -329,23 +298,19 @@ class User(Resource):
         html = mail_utils.renderTemplate('resetPassword.mako', {
             'password': randomPass
         })
-        mail_utils.sendEmail(to=email, subject='Girder: Password reset',
-                             text=html)
+        mail_utils.sendEmail(to=email, subject='Girder: Password reset', text=html)
         self.model('user').setPassword(user, randomPass)
         return {'message': 'Sent password reset email.'}
 
     @access.public
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Create a temporary access token for a user.  The user\'s '
                     'password is not changed.')
-        .param('email', 'Your email address.')
+        .param('email', 'Your email address.', strip=True)
         .errorResponse('That email does not exist in the system.')
     )
-    def generateTemporaryPassword(self, params):
-        self.requireParams('email', params)
-        email = params['email'].lower().strip()
-
-        user = self.model('user').findOne({'email': email})
+    def generateTemporaryPassword(self, email, params):
+        user = self.model('user').findOne({'email': email.lower()})
 
         if not user:
             raise RestException('That email is not registered.')
@@ -360,32 +325,26 @@ class User(Resource):
             'url': url,
             'token': str(token['_id'])
         })
-        mail_utils.sendEmail(to=email, subject='Girder: Temporary access',
-                             text=html)
+        mail_utils.sendEmail(to=email, subject='Girder: Temporary access', text=html)
         return {'message': 'Sent temporary access email.'}
 
     @access.public
-    @loadmodel(model='user', force=True)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Check if a specified token is a temporary access token '
                     'for the specified user.  If the token is valid, returns '
                     'information on the token and user.')
-        .param('id', 'The user ID to check.', paramType='path')
+        .modelParam('id', 'The user ID to check.', model='user', force=True)
         .param('token', 'The token to check.')
-        .errorResponse('The token does not grant temporary access to the '
-                       'specified user.', 401)
+        .errorResponse('The token does not grant temporary access to the specified user.', 401)
     )
-    def checkTemporaryPassword(self, user, params):
-        self.requireParams('token', params)
+    def checkTemporaryPassword(self, user, token, params):
         token = self.model('token').load(
-            params['token'], force=True, objectId=False, exc=True)
+            token, user=user, level=AccessType.ADMIN, objectId=False, exc=True)
         delta = (token['expires'] - datetime.datetime.utcnow()).total_seconds()
-        hasScope = self.model('token').hasScope(
-            token, TokenScope.TEMPORARY_USER_AUTH)
+        hasScope = self.model('token').hasScope(token, TokenScope.TEMPORARY_USER_AUTH)
 
         if token.get('userId') != user['_id'] or delta <= 0 or not hasScope:
-            raise AccessException(
-                'The token does not grant temporary access to this user.')
+            raise AccessException('The token does not grant temporary access to this user.')
 
         # Temp auth is verified, send an actual auth token now. We keep the
         # temp token around since it can still be used on a subsequent request
@@ -403,10 +362,9 @@ class User(Resource):
         }
 
     @access.public
-    @loadmodel(model='user', level=AccessType.READ)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get detailed information about a user.')
-        .param('id', 'The ID of the user.', paramType='path')
+        .modelParam('id', model='user', level=AccessType.READ)
         .errorResponse()
         .errorResponse('Read access was denied on the user.', 403)
     )
@@ -417,20 +375,17 @@ class User(Resource):
         }
 
     @access.public
-    @loadmodel(model='user', force=True)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Verify an email address using a token.')
-        .param('id', 'The user ID to check.', paramType='path')
+        .modelParam('id', 'The user ID to check.', model='user', force=True)
         .param('token', 'The token to check.')
         .errorResponse('The token is invalid or expired.', 401)
     )
-    def verifyEmail(self, user, params):
-        self.requireParams('token', params)
+    def verifyEmail(self, user, token, params):
         token = self.model('token').load(
-            params['token'], force=True, objectId=False, exc=True)
+            token, user=user, level=AccessType.ADMIN, objectId=False, exc=True)
         delta = (token['expires'] - datetime.datetime.utcnow()).total_seconds()
-        hasScope = self.model('token').hasScope(
-            token, TokenScope.EMAIL_VERIFICATION)
+        hasScope = self.model('token').hasScope(token, TokenScope.EMAIL_VERIFICATION)
 
         if token.get('userId') != user['_id'] or delta <= 0 or not hasScope:
             raise AccessException('The token is invalid or expired.')
@@ -440,7 +395,7 @@ class User(Resource):
         user = self.model('user').save(user)
 
         if self.model('user').canLogin(user):
-            setattr(cherrypy.request, 'girderUser', user)
+            setCurrentUser(user)
             authToken = self.sendAuthTokenCookie(user)
             return {
                 'user': self.model('user').filter(user, user),
@@ -458,16 +413,14 @@ class User(Resource):
             }
 
     @access.public
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Send verification email.')
-        .param('login', 'Your login or email address.')
+        .param('login', 'Your login or email address.', strip=True)
         .errorResponse('That login is not registered.', 401)
     )
-    def sendVerificationEmail(self, params):
-        self.requireParams('login', params)
-        login = params['login'].lower().strip()
+    def sendVerificationEmail(self, login, params):
         loginField = 'email' if '@' in login else 'login'
-        user = self.model('user').findOne({loginField: login})
+        user = self.model('user').findOne({loginField: login.lower()})
 
         if not user:
             raise RestException('That login is not registered.', 401)
