@@ -1,8 +1,13 @@
-girder.models.ApiKeyModel = girder.AccessControlledModel.extend({
+import _ from 'underscore';
+
+import AccessControlledModel from 'girder/models/AccessControlledModel';
+import { restRequest } from 'girder/rest';
+
+var ApiKeyModel = AccessControlledModel.extend({
     resourceName: 'api_key',
 
     setActive: function (active) {
-        girder.restRequest({
+        return restRequest({
             path: 'api_key/' + this.id,
             method: 'PUT',
             data: {
@@ -12,16 +17,17 @@ girder.models.ApiKeyModel = girder.AccessControlledModel.extend({
             this.set({active: active});
             this.trigger('g:setActive');
         }, this));
-
-        return this;
     },
 
     save: function () {
         // Scope needs to be sent to the server as JSON
         var scope = this.get('scope');
         this.attributes.scope = JSON.stringify(scope);
-        girder.AccessControlledModel.prototype.save.call(this, arguments);
+        var promise = AccessControlledModel.prototype.save.call(this, arguments);
         // Restore scope to its original state
         this.attributes.scope = scope;
+        return promise;
     }
 });
+
+export default ApiKeyModel;
