@@ -1,10 +1,18 @@
-girder.Router = Backbone.Router.extend({
+import $ from 'jquery';
+import Backbone from 'backbone';
+
+import events from 'girder/events';
+import { parseQueryString } from 'girder/misc';
+
+import 'girder/utilities/jquery/girderModal';
+
+var Router = Backbone.Router.extend({
     initialize: function () {
         this._enabled = true;
     },
 
     execute: function (callback, args) {
-        args.push(girder.parseQueryString(args.pop()));
+        args.push(parseQueryString(args.pop()));
         var queryString = args[args.length - 1];
         if (callback) {
             callback.apply(this, args);
@@ -12,11 +20,11 @@ girder.Router = Backbone.Router.extend({
 
         // handle "top level" dialogs
         if (queryString.dialog === 'login') {
-            girder.events.trigger('g:loginUi');
+            events.trigger('g:loginUi');
         } else if (queryString.dialog === 'register') {
-            girder.events.trigger('g:registerUi');
+            events.trigger('g:registerUi');
         } else if (queryString.dialog === 'resetpassword') {
-            girder.events.trigger('g:resetPasswordUi');
+            events.trigger('g:resetPasswordUi');
         }
     },
 
@@ -38,13 +46,15 @@ girder.Router = Backbone.Router.extend({
     }
 });
 
-girder.router = new girder.Router();
+var router = new Router();
 
 // When the back button is pressed, we want to close open modals.
-girder.router.on('route', function (route, params) {
+router.on('route', function (route, params) {
     if (!params.slice(-1)[0].dialog) {
         $('.modal').girderModal('close');
     }
     // get rid of tooltips
     $('.tooltip').remove();
 });
+
+export default router;
