@@ -9,8 +9,10 @@ import LoadingAnimation from 'girder/views/widgets/LoadingAnimation';
 import SearchFieldWidget from 'girder/views/widgets/SearchFieldWidget';
 import UserModel from 'girder/models/UserModel';
 import View from 'girder/views/View';
+import { getCurrentUser } from 'girder/auth';
 import { AccessType } from 'girder/constants';
 import { handleClose, handleOpen } from 'girder/dialog';
+import { restRequest } from 'girder/rest';
 
 import 'girder/stylesheets/widgets/accessWidget.styl';
 
@@ -27,8 +29,18 @@ var AccessWidget = View.extend({
             $(e.currentTarget).attr('disabled', 'disabled');
             this.saveAccessList();
         },
+        'click .g-close-flags-popover': function (e) {
+            $(e.currentTarget).parents('.g-access-action-container')
+                .find('.g-action-manage-flags').popover('hide');
+        },
+        'click .g-close-public-flags-popover': function (e) {
+            $(e.currentTarget).parents('.g-public-container')
+                .find('.g-action-manage-public-flags').popover('hide');
+        },
         'click a.g-action-remove-access': 'removeAccessEntry',
-        'change .g-public-container .radio input': 'privacyChanged'
+        'change .g-public-container .radio input': 'privacyChanged',
+        'change .g-flag-checkbox': '_toggleAccessFlag',
+        'change .g-public-flag-checkbox': '_togglePublicAccessFlag'
     },
 
     /**
@@ -47,8 +59,12 @@ var AccessWidget = View.extend({
         this.modelType = settings.modelType;
         this.hideRecurseOption = settings.hideRecurseOption || false;
         this.hideSaveButton = settings.hideSaveButton || false;
+        this.hidePrivacyEditor = settings.hidePrivacyEditor || false;
+        this.hideAccessType = settings.hideAccessType || false;
+        this.noAccessFlag = settings.noAccessFlag || false;
         this.modal = _.has(settings, 'modal') ? settings.modal : true;
-
+        this.currentUser = getCurrentUser();
+        this.isAdmin = !!(this.currentUser && this.currentUser.get('admin'));
         this.searchWidget = new SearchFieldWidget({
             placeholder: 'Start typing a name...',
             modes: ['prefix', 'text'],
@@ -56,17 +72,27 @@ var AccessWidget = View.extend({
             parentView: this
         }).on('g:resultClicked', this.addEntry, this);
 
-        if (this.model.get('access')) {
-            this.render();
+        var flagListPromise = null;
+        if (!this.noAccessFlag) {
+            flagListPromise = restRequest({
+                path: 'system/access_flag'
+            }).done((resp) => {
+                this.flagList = resp;
+            });
         } else {
-            this.model.on('g:accessFetched', function () {
-                this.render();
-            }, this).fetchAccess();
+            this.flagList = [];
         }
+
+        $.when(
+            flagListPromise,
+            this.model.fetchAccess()
+        ).then(() => {
+            this.render();
+        });
     },
 
     render: function () {
-        if (!this.model.get('access')) {
+        if (!this.model.get('access') || !this.flagList) {
             new LoadingAnimation({
                 el: this.$el,
                 parentView: this
@@ -89,12 +115,18 @@ var AccessWidget = View.extend({
 
         var template = this.modal ? accessEditorTemplate
                                   : accessEditorNonModalTemplate;
+
         this.$el.html(template({
+            _,
             model: this.model,
             modelType: this.modelType,
             publicFlag: this.model.get('public'),
+            publicFlags: this.model.get('publicFlags'),
             hideRecurseOption: this.hideRecurseOption,
-            hideSaveButton: this.hideSaveButton
+            hideSaveButton: this.hideSaveButton,
+            hidePrivacyEditor: this.hidePrivacyEditor,
+            flagList: this.flagList,
+            isAdmin: this.isAdmin
         }));
 
         if (this.modal) {
@@ -103,8 +135,13 @@ var AccessWidget = View.extend({
 
         _.each(this.model.get('access').groups, function (groupAccess) {
             this.$('#g-ac-list-groups').append(accessEntryTemplate({
+                _,
                 accessTypes: AccessType,
                 type: 'group',
+                flagList: this.flagList,
+                isAdmin: this.isAdmin,
+                hideAccessType: this.hideAccessType,
+                noAccessFlag: this.noAccessFlag,
                 entry: _.extend(groupAccess, {
                     title: groupAccess.name,
                     subtitle: groupAccess.description
@@ -114,8 +151,13 @@ var AccessWidget = View.extend({
 
         _.each(this.model.get('access').users, function (userAccess) {
             this.$('#g-ac-list-users').append(accessEntryTemplate({
+                _,
                 accessTypes: AccessType,
                 type: 'user',
+                flagList: this.flagList,
+                isAdmin: this.isAdmin,
+                hideAccessType: this.hideAccessType,
+                noAccessFlag: this.noAccessFlag,
                 entry: _.extend(userAccess, {
                     title: userAccess.name,
                     subtitle: userAccess.login
@@ -133,10 +175,56 @@ var AccessWidget = View.extend({
     },
 
     _makeTooltips: function () {
-        this.$('.g-action-remove-access').tooltip({
+        this.$('.g-access-action-container a,.g-tooltip').tooltip({
             placement: 'bottom',
             animation: false,
-            delay: {show: 100}
+            delay: { show: 100 }
+        });
+
+        this.$('.g-flag-label span').tooltip({
+            placement: 'bottom'
+        });
+
+        // Re-binding popovers actually breaks them, so we make sure to
+        // only bind ones that aren't already bound.
+        _.each(this.$('.g-action-manage-flags'), el => {
+            if (!$(el).data('bs.popover')) {
+                $(el).popover({
+                    trigger: 'manual',
+                    html: true,
+                    placement: 'left',
+                    viewport: {
+                        selector: 'body',
+                        padding: 10
+                    },
+                    content: function () {
+                        return $(this).parent().find('.g-flags-popover-container').html();
+                    }
+                }).click(function () {
+                    $(this).popover('toggle');
+                });
+            }
+        });
+
+        // Re-binding popovers actually breaks them, so we make sure to
+        // only bind ones that aren't already bound.
+        _.each(this.$('.g-action-manage-public-flags'), el => {
+            if (!$(el).data('bs.popover')) {
+                $(el).popover({
+                    trigger: 'manual',
+                    html: true,
+                    placement: 'right',
+                    viewport: {
+                        selector: 'body',
+                        padding: 10
+                    },
+                    content: function () {
+                        return $(this).parent().find('.g-public-flags-popover-container').html();
+                    }
+                }).click(function () {
+                    $(this).popover('toggle');
+                });
+            }
         });
     },
 
@@ -166,6 +254,7 @@ var AccessWidget = View.extend({
             var model = new UserModel();
             model.set('_id', entry.id).on('g:fetched', function () {
                 this.$('#g-ac-list-users').append(accessEntryTemplate({
+                    _,
                     accessTypes: AccessType,
                     type: 'user',
                     entry: {
@@ -173,7 +262,11 @@ var AccessWidget = View.extend({
                         subtitle: model.get('login'),
                         id: entry.id,
                         level: AccessType.READ
-                    }
+                    },
+                    isAdmin: this.isAdmin,
+                    hideAccessType: this.hideAccessType,
+                    noAccessFlag: this.noAccessFlag,
+                    flagList: this.flagList
                 }));
 
                 this._makeTooltips();
@@ -194,6 +287,7 @@ var AccessWidget = View.extend({
             var model = new GroupModel();
             model.set('_id', entry.id).on('g:fetched', function () {
                 this.$('#g-ac-list-groups').append(accessEntryTemplate({
+                    _,
                     accessTypes: AccessType,
                     type: 'group',
                     entry: {
@@ -201,7 +295,11 @@ var AccessWidget = View.extend({
                         subtitle: model.get('description'),
                         id: entry.id,
                         level: AccessType.READ
-                    }
+                    },
+                    isAdmin: this.isAdmin,
+                    hideAccessType: this.hideAccessType,
+                    noAccessFlag: this.noAccessFlag,
+                    flagList: this.flagList
                 }));
 
                 this._makeTooltips();
@@ -210,45 +308,21 @@ var AccessWidget = View.extend({
     },
 
     saveAccessList: function () {
-        // Rebuild the access list
-        var acList = {
-            users: [],
-            groups: []
-        };
+        var acList = this.getAccessList();
 
-        _.each(this.$('.g-group-access-entry'), function (el) {
-            var $el = $(el);
-            acList.groups.push({
-                name: $el.find('.g-desc-title').html(),
-                id: $el.attr('resourceid'),
-                level: parseInt(
-                    $el.find('.g-access-col-right>select').val(),
-                    10
-                )
-            });
-        }, this);
-
-        _.each(this.$('.g-user-access-entry'), function (el) {
-            var $el = $(el);
-            acList.users.push({
-                login: $el.find('.g-desc-subtitle').html(),
-                name: $el.find('.g-desc-title').html(),
-                id: $el.attr('resourceid'),
-                level: parseInt(
-                    $el.find('.g-access-col-right>select').val(),
-                    10
-                )
-            });
-        }, this);
+        var publicFlags = _.map(this.$('.g-public-flag-checkbox:checked'), checkbox => {
+            return $(checkbox).attr('flag');
+        });
 
         this.model.set({
             access: acList,
-            public: this.$('#g-access-public').is(':checked')
+            public: this.$('#g-access-public').is(':checked'),
+            publicFlags: publicFlags
         });
 
         var recurse = this.$('#g-apply-recursive').is(':checked');
 
-        this.model.off('g:accessListSaved')
+        this.model.off('g:accessListSaved', null, this)
                   .on('g:accessListSaved', function () {
                       if (this.modal) {
                           this.$el.modal('hide');
@@ -263,6 +337,47 @@ var AccessWidget = View.extend({
                   });
     },
 
+    getAccessList: function () {
+        // Rebuild the access list
+        var acList = {
+            users: [],
+            groups: []
+        };
+
+        _.each(this.$('.g-group-access-entry'), function (el) {
+            var $el = $(el);
+            acList.groups.push({
+                name: $el.find('.g-desc-title').html(),
+                id: $el.attr('resourceid'),
+                level: parseInt(
+                    $el.find('.g-access-col-right>select').val(),
+                    10
+                ),
+                flags: _.map($el.find('.g-flag-checkbox:checked'), (checkbox) => {
+                    return $(checkbox).attr('flag');
+                })
+            });
+        }, this);
+
+        _.each(this.$('.g-user-access-entry'), function (el) {
+            var $el = $(el);
+            acList.users.push({
+                login: $el.find('.g-desc-subtitle').html(),
+                name: $el.find('.g-desc-title').html(),
+                id: $el.attr('resourceid'),
+                level: parseInt(
+                    $el.find('.g-access-col-right>select').val(),
+                    10
+                ),
+                flags: _.map($el.find('.g-flag-checkbox:checked'), (checkbox) => {
+                    return $(checkbox).attr('flag');
+                })
+            });
+        }, this);
+
+        return acList;
+    },
+
     removeAccessEntry: function (event) {
         var sel = '.g-user-access-entry,.g-group-access-entry';
         $(event.currentTarget).tooltip('hide').parents(sel).remove();
@@ -272,6 +387,36 @@ var AccessWidget = View.extend({
         this.$('.g-public-container .radio').removeClass('g-selected');
         var selected = this.$('.g-public-container .radio input:checked');
         selected.parents('.radio').addClass('g-selected');
+
+        if (this.$('#g-access-public').is(':checked')) {
+            this.$('.g-action-manage-public-flags').removeClass('hide');
+        } else {
+            this.$('.g-action-manage-public-flags').addClass('hide');
+        }
+    },
+
+    _toggleAccessFlag: function (e) {
+        var el = $(e.currentTarget),
+            type = el.attr('resourcetype'),
+            id = el.attr('resourceid'),
+            flag = el.attr('flag');
+
+        // Since we clicked in a cloned popover element, we must apply this
+        // change within the original element as well.
+        this.$(`.g-flags-popover-container[resourcetype='${type}'][resourceid='${id}']`)
+            .find(`.g-flag-checkbox[flag="${flag}"]`)
+            .attr('checked', el.is(':checked') ? 'checked' : null);
+    },
+
+    _togglePublicAccessFlag: function (e) {
+        var el = $(e.currentTarget),
+            flag = el.attr('flag');
+
+        // Since we clicked in a cloned popover element, we must apply this
+        // change within the original element as well.
+        this.$('.g-public-flags-popover-container')
+            .find(`.g-public-flag-checkbox[flag="${flag}"]`)
+            .attr('checked', el.is(':checked') ? 'checked' : null);
     }
 });
 
