@@ -1,7 +1,24 @@
+import $ from 'jquery';
+import _ from 'underscore';
+
+import EditFileWidget from 'girder/views/widgets/EditFileWidget';
+import FileCollection from 'girder/collections/FileCollection';
+import FileInfoWidget from 'girder/views/widgets/FileInfoWidget';
+import UploadWidget from 'girder/views/widgets/UploadWidget';
+import View from 'girder/views/View';
+import { AccessType } from 'girder/constants';
+import { confirm } from 'girder/dialog';
+import { formatSize } from 'girder/misc';
+import events from 'girder/events';
+
+import FileListTemplate from 'girder/templates/widgets/fileList.pug';
+
+import 'bootstrap/js/tooltip';
+
 /**
  * This widget shows a list of files in a given item.
  */
-girder.views.FileListWidget = girder.View.extend({
+var FileListWidget = View.extend({
     events: {
         'click a.g-show-more-files': function () {
             this.collection.fetchNextPage();
@@ -9,7 +26,7 @@ girder.views.FileListWidget = girder.View.extend({
 
         'click a.g-show-info': function (e) {
             var cid = $(e.currentTarget).attr('file-cid');
-            new girder.views.FileInfoWidget({
+            new FileInfoWidget({
                 el: $('#g-dialog-container'),
                 model: this.collection.get(cid),
                 parentView: this
@@ -30,14 +47,14 @@ girder.views.FileListWidget = girder.View.extend({
             var cid = $(e.currentTarget).parent().attr('file-cid');
             var file = this.collection.get(cid);
 
-            girder.confirm({
+            confirm({
                 text: 'Are you sure you want to delete the file <b>' +
                       file.escape('name') + '</b>?',
                 yesText: 'Delete',
                 escapedHtml: true,
                 confirmCallback: _.bind(function () {
                     file.once('g:deleted', function () {
-                        girder.events.trigger('g:alert', {
+                        events.trigger('g:alert', {
                             icon: 'ok',
                             type: 'success',
                             text: 'File deleted.',
@@ -46,7 +63,7 @@ girder.views.FileListWidget = girder.View.extend({
 
                         this.render();
                     }, this).once('g:error', function () {
-                        girder.events.trigger('g:alert', {
+                        events.trigger('g:alert', {
                             icon: 'cancel',
                             text: 'Failed to delete file.',
                             type: 'danger',
@@ -62,7 +79,7 @@ girder.views.FileListWidget = girder.View.extend({
         this.upload = settings.upload;
         this.fileEdit = settings.fileEdit;
         this.checked = [];
-        this.collection = new girder.collections.FileCollection();
+        this.collection = new FileCollection();
         this.collection.altUrl = 'item/' +
             (settings.itemId || settings.item.get('_id')) + '/files';
         this.collection.append = true; // Append, don't replace pages
@@ -75,7 +92,7 @@ girder.views.FileListWidget = girder.View.extend({
     },
 
     editFileDialog: function (cid) {
-        this.editFileWidget = new girder.views.EditFileWidget({
+        this.editFileWidget = new EditFileWidget({
             el: $('#g-dialog-container'),
             file: this.collection.get(cid),
             parentView: this
@@ -86,14 +103,14 @@ girder.views.FileListWidget = girder.View.extend({
     },
 
     uploadDialog: function (cid) {
-        new girder.views.UploadWidget({
+        new UploadWidget({
             el: $('#g-dialog-container'),
             title: 'Replace file contents',
             parent: this.collection.get(cid),
             parentType: 'file',
             parentView: this
         }).on('g:uploadFinished', function () {
-            girder.events.trigger('g:alert', {
+            events.trigger('g:alert', {
                 icon: 'ok',
                 text: 'File contents updated.',
                 type: 'success',
@@ -104,10 +121,11 @@ girder.views.FileListWidget = girder.View.extend({
 
     render: function () {
         this.checked = [];
-        this.$el.html(girder.templates.fileList({
+        this.$el.html(FileListTemplate({
             files: this.collection.toArray(),
             hasMore: this.collection.hasNextPage(),
-            girder: girder,
+            AccessType: AccessType,
+            formatSize: formatSize,
             parentItem: this.parentItem
         }));
 
@@ -137,3 +155,6 @@ girder.views.FileListWidget = girder.View.extend({
         this.trigger('g:changed');
     }
 });
+
+export default FileListWidget;
+
