@@ -73,6 +73,7 @@ var HierarchyWidget = View.extend({
     events: {
         'click a.g-create-subfolder': 'createFolderDialog',
         'click a.g-edit-folder': 'editFolderDialog',
+        'click button.g-select-folder': 'selectFolder',
         'click a.g-delete-folder': 'deleteFolderDialog',
         'click .g-folder-info-button': 'showInfoDialog',
         'click .g-collection-info-button': 'showInfoDialog',
@@ -117,13 +118,22 @@ var HierarchyWidget = View.extend({
 
         this._showActions = _.has(settings, 'showActions') ? settings.showActions : true;
         this._showItems = _.has(settings, 'showItems') ? settings.showItems : true;
+
+        this._itemFilter = settings.itemFilter;
+
         this._checkboxes = _.has(settings, 'checkboxes') ? settings.checkboxes : true;
+        this._downloadLinks = _.has(settings, 'downloadLinks') ? settings.downloadLinks : true;
+        this._viewLinks = _.has(settings, 'viewLinks') ? settings.viewLinks : true;
+        this._showSizes = _.has(settings, 'showSizes') ? settings.showSizes : true;
+        this._showMetadata = _.has(settings, 'showMetadata') ? settings.showMetadata : true;
         this._routing = _.has(settings, 'routing') ? settings.routing : true;
         this._appendPages = _.has(settings, 'appendPages') ? settings.appendPages : false;
         this._onItemClick = settings.onItemClick || function (item) {
             router.navigate('item/' + item.get('_id'), {trigger: true});
         };
 
+        this._onFolderSelect = settings.onFolderSelect;
+
         this.folderAccess = settings.folderAccess;
         this.folderCreate = settings.folderCreate;
         this.folderEdit = settings.folderEdit;
@@ -150,6 +160,7 @@ var HierarchyWidget = View.extend({
         });
 
         this.folderListView = new FolderListWidget({
+            folderFilter: this._itemFilter,
             parentType: this.parentModel.resourceName,
             parentId: this.parentModel.get('_id'),
             checkboxes: this._checkboxes,
@@ -179,7 +190,9 @@ var HierarchyWidget = View.extend({
         } else {
             this.render();
         }
-        events.on('g:login', this.constructor.resetPickedResources, this);
+        events.on('g:login', () => {
+            this.constructor.resetPickedResources();
+        }, this);
     },
 
     /**
@@ -200,8 +213,13 @@ var HierarchyWidget = View.extend({
      */
     _initFolderViewSubwidgets: function () {
         this.itemListView = new ItemListWidget({
+            itemFilter: this._itemFilter,
             folderId: this.parentModel.get('_id'),
+            accessLevel: this.parentModel.getAccessLevel(),
             checkboxes: this._checkboxes,
+            downloadLinks: this._downloadLinks,
+            viewLinks: this._viewLinks,
+            showSizes: this._showSizes,
             parentView: this
         });
         this.itemListView.on('g:itemClicked', this._onItemClick, this)
@@ -262,9 +280,12 @@ var HierarchyWidget = View.extend({
             model: this.parentModel,
             level: this.parentModel.getAccessLevel(),
             AccessType: AccessType,
+            onFolderSelect: this._onFolderSelect,
             showActions: this._showActions,
+            showMetadata: this._showMetadata,
             checkboxes: this._checkboxes,
-            capitalize: capitalize
+            capitalize: capitalize,
+            itemFilter: this._itemFilter
         }));
 
         if (this.$('.g-folder-actions-menu>li>a').length === 0) {
@@ -280,7 +301,9 @@ var HierarchyWidget = View.extend({
         if (this.parentModel.resourceName === 'folder' && this._showItems) {
             this.itemListView.setElement(this.$('.g-item-list-container')).render();
             this.metadataWidget.setItem(this.parentModel);
-            this.metadataWidget.setElement(this.$('.g-folder-metadata')).render();
+            if (this._showMetadata) {
+                this.metadataWidget.setElement(this.$('.g-folder-metadata')).render();
+            }
         }
 
         this.$('[title]').tooltip({
@@ -327,6 +350,13 @@ var HierarchyWidget = View.extend({
         this.setCurrentModel(this.breadcrumbs[this.breadcrumbs.length - 1]);
     },
 
+    /**
+     * Called when the "select this folder" link is clicked.
+     */
+    selectFolder: function () {
+        this._onFolderSelect(this.parentModel);
+    },
+
     /**
      * Prompt the user to create a new subfolder in the current folder.
      */
@@ -497,7 +527,8 @@ var HierarchyWidget = View.extend({
         this.folderListView.initialize({
             parentType: parent.resourceName,
             parentId: parent.get('_id'),
-            checkboxes: this._checkboxes
+            checkboxes: this._checkboxes,
+            folderFilter: this._itemFilter
         });
 
         this.updateChecked();
@@ -506,7 +537,11 @@ var HierarchyWidget = View.extend({
             if (this.itemListView) {
                 this.itemListView.initialize({
                     folderId: parent.get('_id'),
-                    checkboxes: this._checkboxes
+                    checkboxes: this._checkboxes,
+                    downloadLinks: this._downloadLinks,
+                    viewLinks: this._viewLinks,
+                    itemFilter: this._itemFilter,
+                    showSizes: this._showSizes
                 });
             } else {
                 this._initFolderViewSubwidgets();
