@@ -71,9 +71,11 @@ var RootSelectorWidget = View.extend({
         _.extend(this.groups, settings.groups);
 
         // attach collection change events
-        _.each(this.groups, _.bind(function (group) {
-            this.listenTo(group, 'g:changed', this._updateGroup);
-        }, this));
+        _.each(this.groups, (group) => {
+            this.listenTo(group, 'g:changed', () => {
+                this._updateGroup(group);
+            });
+        });
 
         // possible values that determine rendered behavior for "Home":
         //   - model: show this model as Home
