@@ -284,7 +284,7 @@ class Item(acl_mixin.AccessControlMixin, Model):
         # Validate and save the item
         return self.save(item)
 
-    def setMetadata(self, item, metadata):
+    def setMetadata(self, item, metadata, allowNull=False):
         """
         Set metadata on an item.  A `ValidationException` is thrown in the
         cases where the metadata JSON object is badly formed, or if any of the
@@ -295,6 +295,9 @@ class Item(acl_mixin.AccessControlMixin, Model):
         :param metadata: A dictionary containing key-value pairs to add to
                      the items meta field
         :type metadata: dict
+        :param allowNull: Whether to allow `null` values to be set in the item's
+                     metadata. If set to `False` or omitted, a `null` value will cause that
+                     metadata field to be deleted.
         :returns: the item document
         """
         if 'meta' not in item:
@@ -304,15 +307,43 @@ class Item(acl_mixin.AccessControlMixin, Model):
         item['meta'].update(six.viewitems(metadata))
 
         # Remove metadata fields that were set to null (use items in py3)
-        toDelete = [k for k, v in six.viewitems(item['meta']) if v is None]
-        for key in toDelete:
-            del item['meta'][key]
+        if not allowNull:
+            toDelete = [k for k, v in six.viewitems(metadata) if v is None]
+            for key in toDelete:
+                del item['meta'][key]
+
+        self.validateKeys(item['meta'])
 
         item['updated'] = datetime.datetime.utcnow()
 
         # Validate and save the item
         return self.save(item)
 
+    def deleteMetadata(self, item, fields):
+        """
+        Delete metadata on an item. A `ValidationException` is thrown if the
+        metadata field names contain a period ('.') or begin with a dollar sign
+        ('$').
+
+        :param item: The item to delete metadata from.
+        :type item: dict
+        :param fields: An array containing the field names to delete from the
+            item's meta field
+        :type field: list
+        :returns: the item document
+        """
+        self.validateKeys(fields)
+
+        if 'meta' not in item:
+            item['meta'] = {}
+
+        for field in fields:
+            item['meta'].pop(field, None)
+
+        item['updated'] = datetime.datetime.utcnow()
+
+        return self.save(item)
+
     def parentsToRoot(self, item, user=None, force=False):
         """
         Get the path to traverse to a root of the hierarchy.
@@ -412,7 +443,7 @@ class Item(acl_mixin.AccessControlMixin, Model):
         :type subpath: bool
         :param mimeFilter: Optional list of MIME types to filter by. Set to
             None to include all files.
-        :type mimeFilter: list or tuple
+        :type mimeFilter: `list or tuple`
         :param data: If True return raw content of each file as stored in the
             assetstore, otherwise return file document.
         :type data: bool
