@@ -17,8 +17,6 @@
 #  limitations under the License.
 ###############################################################################
 
-
-import bson.json_util
 import six
 
 from geojson import GeoJSON
@@ -26,8 +24,8 @@ from pymongo import GEOSPHERE
 from pymongo.errors import OperationFailure
 
 from girder.api import access
-from girder.api.describe import Description, describeRoute
-from girder.api.rest import loadmodel, Resource, RestException
+from girder.api.describe import Description, autoDescribeRoute
+from girder.api.rest import Resource, RestException
 from girder.constants import AccessType
 
 
@@ -40,14 +38,12 @@ class GeospatialItem(Resource):
     """
 
     @access.user
-    @describeRoute(
-        Description('Create new items from a GeoJSON feature or feature'
-                    ' collection.')
-        .param('folderId', 'The ID of the parent folder.', required=True,
-               paramType='query')
-        .param('geoJSON', 'A GeoJSON object containing the features or feature'
-                          ' collection to add.', required=True,
-               paramType='query')
+    @autoDescribeRoute(
+        Description('Create new items from a GeoJSON feature or feature collection.')
+        .modelParam('folderId', 'The ID of the parent folder.', model='folder',
+                    level=AccessType.WRITE, paramType='formData')
+        .jsonParam('geoJSON', 'A GeoJSON object containing the features or feature'
+                   ' collection to add.')
         .errorResponse()
         .errorResponse('Invalid GeoJSON was passed in request body.')
         .errorResponse('GeoJSON feature or feature collection was not passed in'
@@ -59,33 +55,16 @@ class GeospatialItem(Resource):
         .notes("All GeoJSON features must contain a property named 'name' from"
                " which the name of each created item is taken.")
     )
-    def create(self, params):
-        """
-        Create new items from a GeoJSON feature or feature collection. All
-        GeoJSON features must contain a property named 'name' from which the
-        name of each created item is taken.
-
-        :param params: parameters to the API call, including 'folderId' and
-                       'geoJSON'.
-        :type params: dict[str, unknown]
-        :returns: filtered fields of the created items with properties appended
-                 to the 'meta' field and geospatial data appended to the 'geo'
-                 field of each item.
-        :rtype: list[dict[str, unknown]]
-        :raise RestException: on malformed, forbidden, or unauthorized API call.
-        """
-        self.requireParams(('folderId', 'geoJSON'), params)
-
+    def create(self, folder, geoJSON, params):
         try:
-            geospatial = bson.json_util.loads(params['geoJSON'])
-            GeoJSON.to_instance(geospatial, strict=True)
+            GeoJSON.to_instance(geoJSON, strict=True)
         except ValueError:
             raise RestException('Invalid GeoJSON passed in request body.')
 
-        if geospatial['type'] == 'Feature':
-            features = [geospatial]
-        elif geospatial['type'] == 'FeatureCollection':
-            features = geospatial['features']
+        if geoJSON['type'] == 'Feature':
+            features = [geoJSON]
+        elif geoJSON['type'] == 'FeatureCollection':
+            features = geoJSON['features']
         else:
             raise RestException('GeoJSON feature or feature collection must be '
                                 'passed in request body.')
@@ -111,8 +90,7 @@ class GeospatialItem(Resource):
                                         ' character long.')
                 if '.' in key or key[0] == '$':
                     raise RestException('The property name %s must not contain'
-                                        ' a period or begin with a dollar'
-                                        ' sign.' % key)
+                                        ' a period or begin with a dollar sign.' % key)
 
             data.append({'name': name,
                          'description': description,
@@ -120,8 +98,6 @@ class GeospatialItem(Resource):
                          'geometry': feature['geometry']})
 
         user = self.getCurrentUser()
-        folder = self.model('folder').load(
-            id=params['folderId'], user=user, level=AccessType.WRITE, exc=True)
 
         items = []
 
@@ -137,75 +113,31 @@ class GeospatialItem(Resource):
         return [self._filter(item) for item in items]
 
     @access.public
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Search for an item by geospatial data.')
-        .param('q', 'Search query as a JSON object.', required=True)
-        .param('limit', 'Result set size limit (default=50).', required=False,
-               dataType='integer')
-        .param('offset', 'Offset into result set (default=0).', required=False,
-               dataType='integer')
+        .jsonParam('q', 'Search query as a JSON object.')
+        .pagingParams(defaultSort='lowerName')
         .errorResponse()
     )
-    def find(self, params):
-        """
-        Search for an item by geospatial data.
-
-        :param params: parameters to the API call, including 'q'.
-        :type params: dict[str, unknown]
-        :returns: filtered fields of the matching items with geospatial data
-                 appended to the 'geo' field of each item.
-        :rtype: list[dict[str, unknown]]
-        :raise RestException: on malformed API call.
-        """
-        self.requireParams(('q',), params)
-
-        try:
-            query = bson.json_util.loads(params['q'])
-        except ValueError:
-            raise RestException("Invalid JSON passed as 'q' parameter.")
-
-        limit, offset, sort = self.getPagingParameters(params, 'lowerName')
-
-        return self._find(query, limit, offset, sort)
+    def find(self, q, limit, offset, sort, params):
+        return self._find(q, limit, offset, sort)
 
     @access.public
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Search for items that intersects with a GeoJSON object.')
-        .param('field', 'Name of field containing GeoJSON on which to search.',
-               required=True)
-        .param('geometry', 'Search query condition as a GeoJSON object.',
-               required=True)
-        .param('limit', 'Result set size limit (default=50).', required=False,
-               dataType='integer')
-        .param('offset', 'Offset into result set (default=0).', required=False,
-               dataType='integer')
+        .param('field', 'Name of field containing GeoJSON on which to search.', strip=True)
+        .jsonParam('geometry', 'Search query condition as a GeoJSON object.')
+        .pagingParams(defaultSort='lowerName')
         .errorResponse()
     )
-    def intersects(self, params):
-        """
-        Search for items that intersects with a GeoJSON object.
-
-        :param params: parameters to the API call, including 'field' and
-                       'geometry'.
-        :type params: dict[str, unknown]
-        :returns: filtered fields of the matching items with geospatial data
-                 appended to the 'geo' field of each item.
-        :rtype: list[dict[str, unknown]]
-        :raise RestException: on malformed API call.
-        """
-        self.requireParams(('field', 'geometry'), params)
-
+    def intersects(self, field, geometry, limit, offset, sort, params):
         try:
-            geometry = bson.json_util.loads(params['geometry'])
             GeoJSON.to_instance(geometry, strict=True)
         except (TypeError, ValueError):
-            raise RestException("Invalid GeoJSON passed as 'geometry'"
-                                " parameter.")
+            raise RestException("Invalid GeoJSON passed as 'geometry' parameter.")
 
-        if params['field'][:3] == '%s.' % GEOSPATIAL_FIELD:
-            field = params['field'].strip()
-        else:
-            field = '%s.%s' % (GEOSPATIAL_FIELD, params['field'].strip())
+        if field[:3] != '%s.' % GEOSPATIAL_FIELD:
+            field = '%s.%s' % (GEOSPATIAL_FIELD, field)
 
         query = {
             field: {
@@ -215,13 +147,10 @@ class GeospatialItem(Resource):
             }
         }
 
-        limit, offset, sort = self.getPagingParameters(params, 'lowerName')
-
         return self._find(query, limit, offset, sort)
 
-    def _getGeometry(self, params):
+    def _getGeometry(self, geometry):
         try:
-            geometry = bson.json_util.loads(params['geometry'])
             GeoJSON.to_instance(geometry, strict=True)
 
             if geometry['type'] != 'Point':
@@ -229,77 +158,45 @@ class GeospatialItem(Resource):
 
             return geometry
         except (TypeError, ValueError):
-            raise RestException("Invalid GeoJSON passed as 'geometry'"
-                                " parameter.")
+            raise RestException("Invalid GeoJSON passed as 'geometry' parameter.")
 
     @access.public
-    @describeRoute(
-        Description('Search for items that are in proximity to a GeoJSON'
-                    ' point.')
-        .param('field', 'Name of field containing GeoJSON on which to search.',
-               required=True)
-        .param('geometry', 'Search query condition as a GeoJSON point.',
-               required=True)
-        .param('maxDistance', 'Limits results to items that are at most this'
-                              ' distance in meters from the GeoJSON point.',
-               required=False, dataType='number')
-        .param('minDistance', 'Limits results to items that are at least this'
-                              ' distance in meters from the GeoJSON point.',
-               required=False, dataType='number')
-        .param('ensureIndex', 'Create a 2dsphere index on the field on which to'
-                              ' search if one does not exist.', required=False,
-               dataType='boolean')
-        .param('limit', 'Result set size limit (default=50).', required=False,
-               dataType='integer')
-        .param('offset', 'Offset into result set (default=0).', required=False,
-               dataType='integer')
+    @autoDescribeRoute(
+        Description('Search for items that are in proximity to a GeoJSON point.')
+        .param('field', 'Name of field containing GeoJSON on which to search.', strip=True)
+        .jsonParam('geometry', 'Search query condition as a GeoJSON point.')
+        .param('maxDistance', 'Limits results to items that are at most this distance '
+               'in meters from the GeoJSON point.', required=False, dataType='number')
+        .param('minDistance', 'Limits results to items that are at least this distance '
+               'in meters from the GeoJSON point.', required=False, dataType='number')
+        .param('ensureIndex', 'Create a 2dsphere index on the field on which to search '
+               'if one does not exist.', required=False, dataType='boolean', default=False)
+        .pagingParams(defaultSort='lowerName')
         .errorResponse()
         .errorResponse('Field on which to search was not indexed.')
         .errorResponse('Index creation was denied.', 403)
         .notes("Field on which to search be indexed by a 2dsphere index."
-               " Anonymous users may not use 'ensureIndex' to create such an"
-               " index.")
+               " Anonymous users may not use 'ensureIndex' to create such an index.")
     )
-    def near(self, params):
-        """
-        Search for items that are in proximity to a GeoJSON point. The field on
-        which to search must be indexed by a '2dsphere' index. Anonymous users
-        may not use 'ensureIndex' to create such an index.
-
-        :param params: parameters to the API call, including 'field' and
-                       'geometry'.
-        :type params: dict[str, unknown]
-        :returns: filtered fields of the matching items with geospatial data
-                 appended to the 'geo' field of each item.
-        :rtype: list[dict[str, unknown]]
-        :raise RestException: on malformed or forbidden API call.
-        """
-        self.requireParams(('field', 'geometry'), params)
-
+    def near(self, field, geometry, maxDistance, minDistance, ensureIndex, limit,
+             offset, sort, params):
         condition = {
-            '$geometry': self._getGeometry(params)
+            '$geometry': self._getGeometry(geometry)
         }
 
-        for param in ('maxDistance', 'minDistance'):
-            if param in params:
-                try:
-                    distance = float(params[param])
+        if maxDistance is not None:
+            if maxDistance < 0:
+                raise RestException('maxDistance must be positive.')
+            condition['$maxDistance'] = maxDistance
+        if minDistance is not None:
+            if minDistance < 0:
+                raise RestException('minDistance must be positive.')
+            condition['$minDistance'] = minDistance
 
-                    if distance < 0.0:
-                        raise ValueError
+        if field[:3] != '%s.' % GEOSPATIAL_FIELD:
+            field = '%s.%s' % (GEOSPATIAL_FIELD, field)
 
-                except ValueError:
-                    raise RestException("Parameter '%s' must be a number." %
-                                        param)
-
-                condition['$'+param] = distance
-
-        if params['field'][:3] == '%s.' % GEOSPATIAL_FIELD:
-            field = params['field'].strip()
-        else:
-            field = '%s.%s' % (GEOSPATIAL_FIELD, params['field'].strip())
-
-        if params.get('ensureIndex', False):
+        if ensureIndex:
             user = self.getCurrentUser()
 
             if not user:
@@ -313,74 +210,48 @@ class GeospatialItem(Resource):
             }
         }
 
-        limit, offset, sort = self.getPagingParameters(params, 'lowerName')
-
         try:
-            items = self._find(query, limit, offset, sort)
+            return self._find(query, limit, offset, sort)
         except OperationFailure:
-            raise RestException("Field '%s' must be indexed by a 2dsphere"
-                                " index." % field)
-
-        return items
+            raise RestException("Field '%s' must be indexed by a 2dsphere index." % field)
 
     _RADIUS_OF_EARTH = 6378137.0  # average in meters
 
     @access.public
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Search for items that are entirely within either a GeoJSON'
                     ' polygon or a circular region.')
-        .param('field', 'Name of field containing GeoJSON on which to search.',
-               required=True)
-        .param('geometry', 'Search query condition as a GeoJSON polygon.',
-               required=False)
-        .param('center', 'Center of search radius as a GeoJSON point.',
-               required=False)
+        .param('field', 'Name of field containing GeoJSON on which to search.', strip=True)
+        .jsonParam('geometry', 'Search query condition as a GeoJSON polygon.',
+                   required=False)
+        .jsonParam('center', 'Center of search radius as a GeoJSON point.',
+                   required=False, requireObject=True)
         .param('radius', 'Search radius in meters.', required=False,
                dataType='number')
-        .param('limit', 'Result set size limit (default=50).', required=False,
-               dataType='integer')
-        .param('offset', 'Offset into result set (default=0).', required=False,
-               dataType='integer')
+        .pagingParams(defaultSort='lowerName')
         .errorResponse()
         .errorResponse('Field on which to search was not indexed.')
         .errorResponse('Index creation was denied.', 403)
         .notes("Either parameter 'geometry' or both parameters 'center' "
                " and 'radius' are required.")
     )
-    def within(self, params):
-        """
-        Search for items that are entirely within either a GeoJSON polygon or a
-        circular region. Either parameter 'geometry' or both parameters 'center'
-        and 'radius' are required.
-
-        :param params: parameters to the API call, including 'field' and either
-                       'geometry' or both 'center' and 'radius'.
-        :type params: dict[str, unknown]
-        :returns: filtered fields of the matching items with geospatial data
-                 appended to the 'geo' field of each item.
-        :rtype: list[dict[str, unknown]]
-        :raise RestException: on malformed API call.
-        """
-        self.requireParams(('field',), params)
-
-        if 'geometry' in params:
+    def within(self, field, geometry, center, radius, limit, offset, sort, params):
+        if geometry is not None:
             try:
-                geometry = bson.json_util.loads(params['geometry'])
                 GeoJSON.to_instance(geometry, strict=True)
 
                 if geometry['type'] != 'Polygon':
                     raise ValueError
             except (TypeError, ValueError):
-                raise RestException("Invalid GeoJSON passed as 'geometry'"
-                                    " parameter.")
+                raise RestException("Invalid GeoJSON passed as 'geometry' parameter.")
 
             condition = {
                 '$geometry': geometry
             }
 
-        elif 'center' in params and 'radius' in params:
+        elif center is not None and radius is not None:
             try:
-                radius = float(params['radius']) / self._RADIUS_OF_EARTH
+                radius /= self._RADIUS_OF_EARTH
 
                 if radius < 0.0:
                     raise ValueError
@@ -388,14 +259,12 @@ class GeospatialItem(Resource):
                 raise RestException("Parameter 'radius' must be a number.")
 
             try:
-                center = bson.json_util.loads(params['center'])
                 GeoJSON.to_instance(center, strict=True)
 
                 if center['type'] != 'Point':
                     raise ValueError
             except (TypeError, ValueError):
-                raise RestException("Invalid GeoJSON passed as 'center'"
-                                    " parameter.")
+                raise RestException("Invalid GeoJSON passed as 'center' parameter.")
 
             condition = {
                 '$centerSphere': [center['coordinates'], radius]
@@ -405,12 +274,8 @@ class GeospatialItem(Resource):
             raise RestException("Either parameter 'geometry' or both parameters"
                                 " 'center' and 'radius' are required.")
 
-        if params['field'][:3] == '%s.' % GEOSPATIAL_FIELD:
-            field = params['field'].strip()
-        else:
-            field = '%s.%s' % (GEOSPATIAL_FIELD, params['field'].strip())
-
-        limit, offset, sort = self.getPagingParameters(params, 'lowerName')
+        if field[:3] != '%s.' % GEOSPATIAL_FIELD:
+            field = '%s.%s' % (GEOSPATIAL_FIELD, field)
 
         query = {
             field: {
@@ -421,57 +286,29 @@ class GeospatialItem(Resource):
         return self._find(query, limit, offset, sort)
 
     @access.public
-    @loadmodel(model='item', level=AccessType.READ)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get an item and its geospatial data by ID.')
-        .param('id', 'The ID of the item.', paramType='path')
+        .modelParam('id', 'The ID of the item.', model='item', level=AccessType.READ)
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied for the item.', 403)
     )
     def getGeospatial(self, item, params):
-        """
-        Get an item and its geospatial data by ID.
-
-        :param item: item to return along with its geospatial data.
-        :type item: dict[str, unknown]
-        :param params: parameters to the API call, unused.
-        :type params: dict[str, unknown]
-        :returns: filtered fields of the item with geospatial data appended to
-                 its 'geo' field.
-        :rtype : dict[str, unknown]
-        :raise RestException: on malformed or forbidden API call.
-        """
         return self._filter(item)
 
     @access.user
-    @loadmodel(model='item', level=AccessType.WRITE)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Set geospatial fields on an item.')
         .notes('Set geospatial fields to null to delete them.')
-        .param('id', 'The ID of the item.', paramType='path')
-        .param('body', 'A JSON object containing the geospatial fields to add.',
-               paramType='body')
+        .modelParam('id', 'The ID of the item.', model='item', level=AccessType.WRITE)
+        .jsonParam('geospatial', 'A JSON object containing the geospatial fields to add.',
+                   paramType='body')
         .errorResponse('ID was invalid.')
         .errorResponse('Invalid JSON was passed in request body.')
         .errorResponse('Geospatial key name was invalid.')
         .errorResponse('Geospatial field did not contain valid GeoJSON.')
         .errorResponse('Write access was denied for the item.', 403)
     )
-    def setGeospatial(self, item, params):
-        """
-        Set geospatial data on an item.
-
-        :param item: item on which to set geospatial data.
-        :type item: dict[str, unknown]
-        :param params: parameters to the API call, unused.
-        :type params: dict[str, unknown]
-        :returns: filtered fields of the item with geospatial data appended to
-                 its 'geo' field.
-        :rtype : dict[str, unknown]
-        :raise RestException: on malformed, forbidden, or unauthorized API call.
-        """
-        geospatial = self.getBodyJson()
-
+    def setGeospatial(self, item, geospatial, params):
         for k, v in six.viewitems(geospatial):
             if '.' in k or k[0] == '$':
                 raise RestException('Geospatial key name %s must not contain a'
@@ -487,8 +324,7 @@ class GeospatialItem(Resource):
             item[GEOSPATIAL_FIELD] = dict()
 
         item[GEOSPATIAL_FIELD].update(six.viewitems(geospatial))
-        keys = [k for k, v in six.viewitems(item[GEOSPATIAL_FIELD])
-                if v is None]
+        keys = [k for k, v in six.viewitems(item[GEOSPATIAL_FIELD]) if v is None]
 
         for key in keys:
             del item[GEOSPATIAL_FIELD][key]
