@@ -18,8 +18,8 @@
 ###############################################################################
 
 from girder.api import access
-from girder.api.describe import Description, describeRoute
-from girder.api.rest import Resource, filtermodel, loadmodel
+from girder.api.describe import Description, autoDescribeRoute
+from girder.api.rest import Resource, filtermodel
 from girder.constants import AccessType, SortDir
 
 
@@ -29,13 +29,14 @@ class Job(Resource):
         self.resourceName = 'job'
 
         self.route('GET', (), self.listJobs)
+        self.route('GET', ('all',), self.listAllJobs)
         self.route('GET', (':id',), self.getJob)
         self.route('PUT', (':id',), self.updateJob)
         self.route('DELETE', (':id',), self.deleteJob)
 
     @access.public
     @filtermodel(model='job', plugin='jobs')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('List jobs for a given user.')
         .param('userId', 'The ID of the user whose jobs will be listed. If '
                'not passed or empty, will use the currently logged in user. If '
@@ -43,40 +44,44 @@ class Job(Resource):
                'user.', required=False)
         .pagingParams(defaultSort='created', defaultSortDir=SortDir.DESCENDING)
     )
-    def listJobs(self, params):
-        limit, offset, sort = self.getPagingParameters(
-            params, 'created', SortDir.DESCENDING)
+    def listJobs(self, userId, limit, offset, sort, params):
         currentUser = self.getCurrentUser()
-        userId = params.get('userId')
         if not userId:
             user = currentUser
         elif userId.lower() == 'none':
             user = None
         else:
-            user = self.model('user').load(
-                params['userId'], user=currentUser, level=AccessType.READ)
+            user = self.model('user').load(userId, user=currentUser, level=AccessType.READ)
 
         return list(self.model('job', 'jobs').list(
-            user=user, offset=offset, limit=limit, sort=sort,
-            currentUser=currentUser))
+            user=user, offset=offset, limit=limit, sort=sort, currentUser=currentUser))
+
+    @access.admin
+    @filtermodel(model='job', plugin='jobs')
+    @autoDescribeRoute(
+        Description('List all jobs.')
+        .pagingParams(defaultSort='created', defaultSortDir=SortDir.DESCENDING)
+    )
+    def listAllJobs(self, limit, offset, sort, params):
+        currentUser = self.getCurrentUser()
+        return list(self.model('job', 'jobs').listAll(
+            offset=offset, limit=limit, sort=sort, currentUser=currentUser))
 
     @access.public
-    @loadmodel(model='job', plugin='jobs', level=AccessType.READ,
-               includeLog=True)
-    @describeRoute(
+    @filtermodel(model='job', plugin='jobs')
+    @autoDescribeRoute(
         Description('Get a job by ID.')
-        .param('id', 'The ID of the job.', paramType='path')
+        .modelParam('id', 'The ID of the job.', model='job', plugin='jobs', level=AccessType.READ,
+                    includeLog=True)
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied for the job.', 403)
     )
-    @filtermodel(model='job', plugin='jobs')
     def getJob(self, job, params):
         return job
 
     @access.token
-    @loadmodel(model='job', plugin='jobs', force=True)
     @filtermodel(model='job', plugin='jobs')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Update an existing job.')
         .notes('In most cases, regular users should not call this endpoint. It '
                'will typically be used by a batch processing system to send '
@@ -84,29 +89,27 @@ class Job(Resource):
                'user-associated token for authorization, the token must be '
                'granted the "jobs.job_<id>" scope, where <id> is the ID of '
                'the job being updated.')
-        .param('id', 'The ID of the job.', paramType='path')
+        .modelParam('id', 'The ID of the job.', model='job', plugin='jobs', force=True)
         .param('log', 'A message to add to the job\'s log field. If you want '
                'to overwrite any existing log content, pass another parameter '
                '"overwrite=true".', required=False)
         .param('overwrite', 'If passing a log parameter, you may set this to '
                '"true" if you wish to overwrite the log field rather than '
-               'append to it. The default behavior is to append',
-               dataType='boolean', required=False)
+               'append to it.', dataType='boolean', required=False, default=False)
         .param('status', 'Update the status of the job. See the JobStatus '
                'enumeration in the constants module in this plugin for the '
-               'numerical values of each status.', dataType='integer',
-               required=False)
+               'numerical values of each status.', required=False)
         .param('progressTotal', 'Maximum progress value, set <= 0 to indicate '
-               'indeterminate progress for this job.', required=False)
-        .param('progressCurrent', 'Current progress value.', required=False)
+               'indeterminate progress for this job.', required=False, dataType='float')
+        .param('progressCurrent', 'Current progress value.', required=False, dataType='float')
         .param('progressMessage', 'Current progress message.', required=False)
         .param('notify', 'If this update should trigger a notification, set '
-               'this field to true. The default is true.', dataType='boolean',
-               required=False)
+               'this field to true.', dataType='boolean', required=False, default=True)
         .errorResponse('ID was invalid.')
         .errorResponse('Write access was denied for the job.', 403)
     )
-    def updateJob(self, job, params):
+    def updateJob(self, job, log, overwrite, notify, status, progressTotal, progressCurrent,
+                  progressMessage, params):
         user = self.getCurrentUser()
         if user:
             self.model('job', 'jobs').requireAccess(job, user, level=AccessType.WRITE)
@@ -114,18 +117,14 @@ class Job(Resource):
             self.ensureTokenScopes('jobs.job_' + str(job['_id']))
 
         return self.model('job', 'jobs').updateJob(
-            job, log=params.get('log'), status=params.get('status'),
-            overwrite=self.boolParam('overwrite', params, False),
-            notify=self.boolParam('notify', params, default=True),
-            progressCurrent=params.get('progressCurrent'),
-            progressTotal=params.get('progressTotal'),
-            progressMessage=params.get('progressMessage'))
+            job, log=log, status=status, overwrite=overwrite, notify=notify,
+            progressCurrent=progressCurrent, progressTotal=progressTotal,
+            progressMessage=progressMessage)
 
     @access.user
-    @loadmodel(model='job', plugin='jobs', level=AccessType.ADMIN)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Delete an existing job.')
-        .param('id', 'The ID of the job.', paramType='path')
+        .modelParam('id', 'The ID of the job.', model='job', plugin='jobs', level=AccessType.ADMIN)
         .errorResponse('ID was invalid.')
         .errorResponse('Admin access was denied for the job.', 403)
     )
