@@ -34,7 +34,7 @@ class Job(AccessControlledModel):
             ('userId', SortDir.ASCENDING),
             ('created', SortDir.DESCENDING)
         )
-        self.ensureIndices([(compoundSearchIndex, {})])
+        self.ensureIndices([(compoundSearchIndex, {}), 'created'])
 
         self.exposeFields(level=AccessType.READ, fields={
             'title', 'type', 'created', 'interval', 'when', 'status',
@@ -71,6 +71,22 @@ class Job(AccessControlledModel):
                                                 limit=limit, offset=offset):
             yield r
 
+    def listAll(self, limit=0, offset=0, sort=None, currentUser=None):
+        """
+        List all jobs.
+
+        :param limit: The page limit.
+        :param offset: The page offset
+        :param sort: The sort field.
+        :param currentUser: User for access filtering.
+        """
+        cursor = self.find({}, sort=sort)
+
+        for r in self.filterResultsByPermission(cursor=cursor, user=currentUser,
+                                                level=AccessType.READ,
+                                                limit=limit, offset=offset):
+            yield r
+
     def cancelJob(self, job):
         """
         Revoke/cancel a job. This simply triggers the jobs.cancel event and
