@@ -1,7 +1,9 @@
 import JobDetailsWidget from './JobDetailsWidget';
 import JobListWidget from './JobListWidget';
+import CheckBoxMenu from './CheckBoxMenu';
 
 export {
   JobDetailsWidget,
-  JobListWidget
+  JobListWidget,
+  CheckBoxMenu
 };
