@@ -18,8 +18,8 @@
 ###############################################################################
 
 from girder.api import access
-from girder.api.describe import Description, describeRoute
-from girder.api.rest import filtermodel, loadmodel, Resource, RestException
+from girder.api.describe import Description, autoDescribeRoute
+from girder.api.rest import filtermodel, Resource, RestException
 from girder.constants import AccessType
 
 
@@ -30,55 +30,44 @@ class Thumbnail(Resource):
         self.route('POST', (), self.createThumbnail)
 
     @access.user
-    @loadmodel(map={'fileId': 'file'}, model='file', level=AccessType.READ)
     @filtermodel(model='job', plugin='jobs')
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Create a new thumbnail from an existing image file.')
         .notes('Setting a width or height parameter of 0 will preserve the '
                'original aspect ratio.')
-        .param('fileId', 'The ID of the source file.')
-        .param('width', 'The desired width.', required=False, dataType='int')
-        .param('height', 'The desired height.', required=False, dataType='int')
+        .modelParam('fileId', 'The ID of the source file.', model='file', paramType='formData',
+                    level=AccessType.READ)
+        .param('width', 'The desired width.', required=False, dataType='integer', default=0)
+        .param('height', 'The desired height.', required=False, dataType='integer', default=0)
         .param('crop', 'Whether to crop the image to preserve aspect ratio. '
                'Only used if both width and height parameters are nonzero.',
                dataType='boolean', required=False, default=True)
         .param('attachToId', 'The lifecycle of this thumbnail is bound to the '
                'resource specified by this ID.')
-        .param('attachToType', 'The type of resource to which this thumbnail is'
-               ' attached.', enum=['folder', 'user', 'collection', 'item'])
+        .param('attachToType', 'The type of resource to which this thumbnail is attached.',
+               enum=['folder', 'user', 'collection', 'item'])
         .errorResponse()
         .errorResponse(('Write access was denied on the attach destination.',
                         'Read access was denied on the file.'), 403)
     )
-    def createThumbnail(self, file, params):
-        self.requireParams(('attachToId', 'attachToType'), params)
-
+    def createThumbnail(self, file, width, height, crop, attachToId, attachToType, params):
         user = self.getCurrentUser()
-        width = params.get('width')
-        height = params.get('height')
 
-        if params['attachToType'] not in (
-                'item', 'collection', 'user', 'folder'):
-            raise RestException('You can only attach thumbnails to users, '
-                                'folders, collections, or items.')
+        self.model(attachToType).load(attachToId, user=user, level=AccessType.WRITE, exc=True)
 
-        self.model(params['attachToType']).load(
-            params['attachToId'], user=user, level=AccessType.WRITE, exc=True)
-
-        width = max(int(params.get('width', 0)), 0)
-        height = max(int(params.get('height', 0)), 0)
+        width = max(width, 0)
+        height = max(height, 0)
 
         if not width and not height:
-            raise RestException(
-                'You must specify a valid width, height, or both.')
+            raise RestException('You must specify a valid width, height, or both.')
 
         kwargs = {
             'width': width,
             'height': height,
             'fileId': str(file['_id']),
-            'crop': self.boolParam('crop', params, default=True),
-            'attachToType': params['attachToType'],
-            'attachToId': params['attachToId']
+            'crop': crop,
+            'attachToType': attachToType,
+            'attachToId': attachToId
         }
 
         job = self.model('job', 'jobs').createLocalJob(
