@@ -17,6 +17,7 @@
 #  limitations under the License.
 ###############################################################################
 
+from bson.objectid import ObjectId
 import functools
 import six
 import sys
@@ -115,8 +116,9 @@ def createThumbnail(width, height, crop, fileId, attachToType, attachToId):
     out.seek(0)
 
     thumbnail = uploadModel.uploadFromFile(
-        out, size=size, name='_thumb.jpg', parentType=None, parent=None,
-        user=None, mimeType='image/jpeg')
+        out, size=size, name='_thumb.jpg', parentType=attachToType,
+        parent={'_id': ObjectId(attachToId)}, user=None, mimeType='image/jpeg',
+        attachParent=True)
 
     return attachThumbnail(
         file, thumbnail, attachToType, attachToId, width, height)
