@@ -17,16 +17,15 @@
 #  limitations under the License.
 ###############################################################################
 
-import json
 import six
 
 from bson.objectid import ObjectId, InvalidId
 from girder import logger
 from girder.api import access
-from girder.api.describe import Description, describeRoute
-from girder.api.rest import Resource, RestException, loadmodel
+from girder.api.describe import Description, autoDescribeRoute
+from girder.api.rest import Resource, RestException
 from girder.constants import AccessType
-from girder.models.model_base import GirderException
+from girder.models.model_base import GirderException, ValidationException
 from girder.utility import assetstore_utilities
 from girder.utility.system import formatSize
 from . import constants
@@ -76,7 +75,7 @@ class QuotaPolicy(Resource):
         filtered[QUOTA_FIELD] = resource.get(QUOTA_FIELD, {})
         return filtered
 
-    def _setResourceQuota(self, model, resource, params):
+    def _setResourceQuota(self, model, resource, policy):
         """
         Handle setting quota policies for any resource that supports them.
 
@@ -85,8 +84,7 @@ class QuotaPolicy(Resource):
         :param params: the query parameters.  'policy' is required and used.
         :returns: the updated resource document.
         """
-        self.requireParams(('policy', ), params)
-        policy = self._validatePolicy(params['policy'])
+        policy = self._validatePolicy(policy)
         if QUOTA_FIELD not in resource:
             resource[QUOTA_FIELD] = {}
         resource[QUOTA_FIELD].update(policy)
@@ -169,13 +167,6 @@ class QuotaPolicy(Resource):
                            dictionary as if the JSON was already decoded.
         :returns: a validate policy dictionary.
         """
-        if not isinstance(policy, dict):
-            try:
-                policy = json.loads(policy)
-            except ValueError:
-                raise RestException('The policy parameter must be JSON.')
-        if not isinstance(policy, dict):
-            raise RestException('The policy parameter must be a dictionary.')
         validKeys = []
         for key in dir(self):
             if key.startswith('_validate_'):
@@ -186,17 +177,16 @@ class QuotaPolicy(Resource):
         for key in policy:
             if key not in validKeys:
                 raise RestException(
-                    '%s is not a valid quota policy key.  Valid keys are '
-                    '%s.' % (key, ', '.join(sorted(validKeys))))
+                    '%s is not a valid quota policy key.  Valid keys are %s.' %
+                    (key, ', '.join(sorted(validKeys))))
             funcName = '_validate_' + key
             policy[key] = getattr(self, funcName)(policy[key])
         return policy
 
     @access.public
-    @loadmodel(model='collection', level=AccessType.READ)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get quota and assetstore policies for the collection.')
-        .param('id', 'The collection ID', paramType='path')
+        .modelParam('id', 'The collection ID', model='collection', level=AccessType.READ)
         .errorResponse('ID was invalid.')
         .errorResponse('Read permission denied on the collection.', 403)
     )
@@ -204,52 +194,47 @@ class QuotaPolicy(Resource):
         if QUOTA_FIELD not in collection:
             collection[QUOTA_FIELD] = {}
         collection[QUOTA_FIELD][
-            '_currentFileSizeQuota'] = self._getFileSizeQuota(
-            'collection', collection)
+            '_currentFileSizeQuota'] = self._getFileSizeQuota('collection', collection)
         return self._filter('collection', collection)
 
     @access.public
-    @loadmodel(model='collection', level=AccessType.ADMIN)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Set quota and assetstore policies for the collection.')
-        .param('id', 'The collection ID', paramType='path')
-        .param('policy', 'A JSON object containing the policies.  This is a '
-               'dictionary of keys and values.  Any key that is not specified '
-               'does not change.', required=True)
+        .modelParam('id', 'The collection ID', model='collection', level=AccessType.ADMIN)
+        .jsonParam('policy', 'A JSON object containing the policies. This is a '
+                   'dictionary of keys and values. Any key that is not specified '
+                   'does not change.', requireObject=True)
         .errorResponse('ID was invalid.')
         .errorResponse('Read permission denied on the collection.', 403)
     )
-    def setCollectionQuota(self, collection, params):
-        return self._setResourceQuota('collection', collection, params)
+    def setCollectionQuota(self, collection, policy, params):
+        return self._setResourceQuota('collection', collection, policy)
 
     @access.public
-    @loadmodel(model='user', level=AccessType.READ)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Get quota and assetstore policies for the user.')
-        .param('id', 'The user ID', paramType='path')
+        .modelParam('id', 'The user ID', model='user', level=AccessType.READ)
         .errorResponse('ID was invalid.')
         .errorResponse('Read permission denied on the user.', 403)
     )
     def getUserQuota(self, user, params):
         if QUOTA_FIELD not in user:
             user[QUOTA_FIELD] = {}
-        user[QUOTA_FIELD]['_currentFileSizeQuota'] = self._getFileSizeQuota(
-            'user', user)
+        user[QUOTA_FIELD]['_currentFileSizeQuota'] = self._getFileSizeQuota('user', user)
         return self._filter('user', user)
 
     @access.public
-    @loadmodel(model='user', level=AccessType.ADMIN)
-    @describeRoute(
+    @autoDescribeRoute(
         Description('Set quota and assetstore policies for the user.')
-        .param('id', 'The user ID', paramType='path')
-        .param('policy', 'A JSON object containing the policies.  This is a '
-               'dictionary of keys and values.  Any key that is not specified '
-               'does not change.', required=True)
+        .modelParam('id', 'The user ID', model='user', level=AccessType.ADMIN)
+        .jsonParam('policy', 'A JSON object containing the policies.  This is a '
+                   'dictionary of keys and values.  Any key that is not specified '
+                   'does not change.', requireObject=True)
         .errorResponse('ID was invalid.')
         .errorResponse('Read permission denied on the user.', 403)
     )
-    def setUserQuota(self, user, params):
-        return self._setResourceQuota('user', user, params)
+    def setUserQuota(self, user, policy, params):
+        return self._setResourceQuota('user', user, policy)
 
     def _checkAssetstore(self, assetstoreSpec):
         """
@@ -411,14 +396,14 @@ class QuotaPolicy(Resource):
             return
         quotaInfo = self._checkUploadSize(event.info)
         if quotaInfo:
-            raise GirderException(
+            raise ValidationException(
                 'Upload would exceed file storage quota (need %s, only %s '
                 'available - used %s out of %s)' %
                 (formatSize(quotaInfo['sizeNeeded']),
                  formatSize(quotaInfo['quotaLeft']),
                  formatSize(quotaInfo['quotaUsed']),
                  formatSize(quotaInfo['fileSizeQuota'])),
-                'user_quota.upload-exceeds-quota')
+                field='size')
 
     def checkUploadFinalize(self, event):
         """
@@ -432,11 +417,11 @@ class QuotaPolicy(Resource):
         if quotaInfo:
             # Delete the upload
             self.model('upload').cancelUpload(upload)
-            raise GirderException(
+            raise ValidationException(
                 'Upload exceeded file storage quota (need %s, only %s '
                 'available - used %s out of %s)' %
                 (formatSize(quotaInfo['sizeNeeded']),
                  formatSize(quotaInfo['quotaLeft']),
                  formatSize(quotaInfo['quotaUsed']),
                  formatSize(quotaInfo['fileSizeQuota'])),
-                'user_quota.upload-exceeds-quota')
+                field='size')
