@@ -22,3 +22,10 @@ DOCKER_DATA_VOLUME = '/mnt/girder_worker/data'
 
 # The path that will be mounted in docker containers for utility scripts
 DOCKER_SCRIPTS_VOUME = '/mnt/girder_worker/scripts'
+
+
+# Settings where plugin information is stored
+class PluginSettings(object):
+    BROKER = 'worker.broker'
+    BACKEND = 'worker.backend'
+    API_URL = 'worker.api_url'
