@@ -1,6 +1,9 @@
-import JobStatus from 'girder_plugins/jobs/JobStatus';
+// Since plugins are not dynamically linked against other plugin libraries,
+// we have to modify the runtime global JobStatus, rather than importing it
+// here statically, which would only modify a local copy.
 
-JobStatus.registerStatus({
+/*global girder*/
+girder.plugins.jobs.JobStatus.registerStatus({
     WORKER_FETCHING_INPUT: {
         value: 820,
         text: 'Fetching input',
